/******************************************************************************    
 * FILE PURPOSE:     - LED parser
 ******************************************************************************    
 * FILE NAME:     ledcfg.c                                                       
 *                                                                                 
 * DESCRIPTION:  LED parser
 *                                                                                 
 * (C) Copyright 2005, Texas Instruments, Inc                                      
 *******************************************************************************/  

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#ifndef __WIN32__
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <asm/avalanche/generic/led_drv_user.h>
#include <asm/avalanche/generic/led_manager.h>
#include "ledcfg.h"
#else
#include "led_drv_user.h"
#include "ledcfg.h"
#endif

#define MAX_KEYWORDS    8
#define MAX_STATES      (MAX_MODULE_ENTRIES * MAX_STATE_ENTRIES)


#define LED_DEV_FILE  "/dev/led"
#define CONF_FILE  "/etc/led.conf"

int read_mod(char *line);
int read_instance(char *line);
int read_state(char *line);
int read_gpio(char *line);
int read_mode(char *line);
int read_param1(char *line);
int read_param2(char *line);
int read_color(char *line);

typedef struct config_kw_handler{
  char *name;
  int (*handler)(char *line);
}config_kw_handler_t;

config_kw_handler_t kwArr[] = 
{
  {"module",read_mod},
  {"instance",read_instance},
  {"state",read_state},
  {"gpio",read_gpio},
  {"mode",read_mode},
  {"param1",read_param1},
  {"param2",read_param2},
  {"color",read_color},
};



char  curModule[MODULE_NAME_LEN];
unsigned char cbfModule[MAX_CBFS_PER_STATE][MAX_CBF_NAME_LEN];
unsigned int cbf_gpio_num;
int curInstance=0;
int gpio_count;
char curName[15];


struct led_config *pTemp;
struct led_config *ptrArr[MAX_STATES];
int ptrIndex = 0;

char arr[255];

int read_mod(char *line)
{
  /* Remember the new module's name */
  strcpy( curModule,line );
  memset( cbfModule, 0, sizeof( cbfModule ) );
  cbf_gpio_num = 0;
  curInstance = 0;
  return 1;
}



int read_instance(char *line)
{
  /* Remember the new instance id */
  unsigned int i;
  i = atoi( line );

  if( i < MAX_MODULE_INSTANCES )
  {
     curInstance = i;
     return 1;
  }
  else
  {
     fprintf( stderr, "%sinvalid module instance id\n", "ledcfg: " );
     return 0;
  }

}




int read_state(char *line)
{

  unsigned int i;

  /* Malloc memory for new state info */
  pTemp = malloc( sizeof(struct led_config) );
  if( pTemp == NULL )
     return 0;
  /* Remember the pointer */
  ptrArr[ptrIndex++] = (struct led_config *)pTemp;
  memset( pTemp,0x00,sizeof(struct led_config) );

  /* save the module name,instance and state var */
  strcpy( pTemp->name,curModule );
  pTemp->instance=curInstance;


  i = atoi( line );

  if( i < MAX_STATE_ENTRIES )
  { 
     pTemp->state = i;
     pTemp->gpio_num = 0;
  }
  else
  {
     fprintf( stderr, "%sinvalid module state id\n", "ledcfg: " );
     return 0;
  }
  return 1;
}

int read_gpio(char *line)
{
  unsigned int i = 0;
  unsigned char cbf = 0;

  if( line[ 0 ] >= '0' && line[ 0 ] <= '9' )
     /* Save the LED number */
     i = atoi( line );
  else
  {
     unsigned int j, found = 0;

     cbf = 1;

     for( j = 0; j < cbf_gpio_num; j++ )
     {
        if( !strcmp( cbfModule[ j ], line ) )
        {
           i = (unsigned char)(~0) - j; 
           found = 1;
           break;
        }
     }

     if( !found )
     {
        i = (unsigned char)(~0) - cbf_gpio_num; 
        strncpy( cbfModule[ cbf_gpio_num++ ], line, MAX_CBF_NAME_LEN );
     }
  }

  if( ( i < MAX_GPIO_PIN_NUM && pTemp->gpio_num < MAX_GPIOS_PER_STATE ) || cbf )
  { 
     pTemp->gpio[pTemp->gpio_num++] = i;
     return 1;
  }
  else
  {
     fprintf( stderr, "%sinvalid gpio id/count\n", "ledcfg: " );
     return 0;
  }
}

int read_mode(char *line)
{

  unsigned int i;

  i = atoi( line );

  if( i >= NUM_LED_MODES )
  {
     printf( " invalid LED mode\n" );
     return 0;
  }  

  /* Save the mode */
  if( pTemp->gpio_num >= 1 )
     pTemp->mode = i;
  return 1;
}

int read_param1(char *line)
{
  pTemp->param1 = atoi( line );
  return 1;
}



int read_param2(char *line)
{
  pTemp->param2 = atoi( line );
  return 1;
}


int read_color(char *line)
{
  unsigned int i;
  i = atoi( line );

  if( i > LED_MAX_COLORS )
     return 0;
  else
     pTemp->color = i;
  return 1;
}


int read_config(char *file)
{
  FILE *in;
  char buffer[80], orig[80], *token, *line;
  int i, j, index;


  if ( !(in = fopen( file, "r" )) ) {
     fprintf( stderr, "%sunable to open config file\n", "ledcfg: " );
     return 0;
  }
  index = -1;	
  while ( fgets( buffer, 80, in ) ) {
     if ( strchr( buffer, '\n' ) ) *(strchr( buffer, '\n' )) = '\0';
     strncpy( orig, buffer, 80 );
     if ( strchr( buffer, '#' ) ) *(strchr( buffer, '#' )) = '\0';
     token = buffer + strspn( buffer, " \t" );
     if ( *token == '\0' ) continue;
     line = token + strcspn( token, " \t=" );
     if ( *line == '\0' ) continue;
     *line = '\0';
     line++;

     /* eat leading whitespace */
     line = line + strspn( line, " \t=" );
     /* eat trailing whitespace */
     for ( i = strlen( line ) ; i > 0 && isspace( line[i-1] ); i-- );
     line[i] = '\0';

     for( j=0 ; j<MAX_KEYWORDS ; j++ )
     {
#ifndef __WIN32__			
        if ( !strcasecmp( token, kwArr[j].name ) )
#else
           if( !strcmp( token,kwArr[j].name ) )
#endif
           {
              if ( !kwArr[j].handler( line ) )
              {
                 fprintf( stderr, "%sunable to parse %s\n", "ledcfg: ",line );
                 return 0;
              }
           }
     }
  }
  fclose( in );
  return 1;
}

int main(int argc, char *argv[])
{
  int i;

#ifndef __WIN32__
  int fp;
#endif

  if( argc > 3 )
  {
     printf( "\nledcfg usage:   ledcfg -f <configuration file path>  \n" );
     return -1;
  }
  else
  {
     if( (argc == 3) && ( strcmp( argv[1] , "-f" ) == 0)  && ( argv[2] != NULL) )
     {
        printf( "%sreading LED config file %s \n", "ledcfg: ",argv[2] );
        read_config( argv[2] );
     }
     else if( argc == 1 )
     {
        printf( "%sreading Standard Configuration File %s\n", "ledcfg: ",CONF_FILE );
        read_config( CONF_FILE );
     }
     else
     {
        printf( "\nledcfg usage:   ledcfg -f <configuration file path>  \n" );
        return -1;
     }
  }


#ifndef __WIN32__
  if ( (fp = open( LED_DEV_FILE, O_RDWR )) == -1 ) {
     fprintf( stderr, "%sunable to open LED device file\n", "ledcfg: " );
     return 0;
  }

  /* Configure element through ioctl system calls */
  for ( i = 0 ; i < ptrIndex ; i++ )
  {
     struct led_config *temp=ptrArr[i];

     ioctl( fp,LED_MANAGER_CONFIG,(int)temp );
  }

  printf( "%sconfigured %d states\n", "ledcfg: ",ptrIndex );

  close( fp );  
#else
  /* Configure element through ioctl system calls */
  for ( i = 0 ; i < ptrIndex ; i++ )
  {
     struct led_config *temp=ptrArr[i];

     printf( "\nLED Configuration: " );
     printf( "name: %s,",   temp->name );
     printf( "instance: %d\n", temp->instance );
     printf( "state: %d,", temp->state );
     printf( "gpio_num: %d,", temp->gpio_num );
     if ( temp->gpio_num > 1 )
        printf( "gpios: %d+%d,", temp->gpio[0], temp->gpio[1] );
     else
        printf( "gpio: %d,", temp->gpio[0] );

     printf( "mode: %d,", temp->mode[0] );
     printf( "param1: %d,", temp->param1 );
     printf( "param2: %d,", temp->param2 );
     printf( "color: %d\n", temp->color );
  }

  printf( "\nConfigured %d states \n",ptrIndex );


#endif

  return 0;
}


