/***************************************************************

        bwbasic.h       Header File
                        for Bywater BASIC Interpreter

    bwBASIC version 3.03
    Copyright (c) 1992-2008, Ted A. Campbell
    Incorporatng code by Jon B. Volkoff
    Incorporatng code by thechief (www.RouterTech.Org)

    http://bwbasic.sourceforge.net
    email: tc at tedcampbell dot com

    Copyright and Permissions Information:

    All U.S. and international rights are claimed by the authors.

    This software is released under the terms of the GNU General
    Public License (GPL), which is distributed with this software
    in the file "COPYING".  The GPL specifies the terms under
    which users may copy and use the software in this distribution.
    Please note that it rules out the distribution of proprietary
    versions of the software, and it rules out the use of altered
    versions of the software. This means that you cannot remove
    the Bywater BASIC name or copyright information.

    A separate license is available for proprietary or commercial
    distribution, for information on which you should contact
    the author.

***************************************************************/

/*---------------------------------------------------------------*/
/* NOTE: Modifications marked "JBV" were made by Jon B. Volkoff, */
/* 11/1995 (eidetics@cerf.net).                                  */
/*---------------------------------------------------------------*/

#ifndef TRUE
#define TRUE    -1
#define FALSE   0
#endif

/***************************************************************

	bwbasic.h	Part I: Definitions

***************************************************************/

/* Version number */
#define VERSION "3.03 (RouterTech.Org development version)"    /* Current version number */

#include "bwx_tty.h"

#define	HAVE_SIGNAL	TRUE
#define	HAVE_LONGJUMP	TRUE

/***************************************************************

	bwbasic.h:	Part I-C: Define Program Configuration

			You must specify one and only one of the
			following progrm configurations as TRUE.
			If you specify CFG_CUSTOM, then you will
			need to fill out the custom section below.

***************************************************************/

/***************************************************************

	bwbasic.h:	Part I-E: Define Natural Language for Messages

			One and only one of the following must be
			defined as TRUE.  Note that the language
			definitions themselves are in file bwb_mes.h.
			If none is specified, then ENGLISH will be
			taken as the default.

***************************************************************/

#define STD_ENGLISH     TRUE            /* standard English */
#define POL_ENGLISH     FALSE 		/* polite English messages */
#define IMP_ENGLISH	FALSE		/* impolite English messages */
#define LATIN		FALSE		/* Latin language messages */
#define STD_RUSSIAN	FALSE		/* Russian language messages */
#define STD_GERMAN      FALSE           /* German language messages */
#define ESPERANTO       FALSE           /* Esperanto messages */

/***************************************************************

	bwbasic.h:	Part I-F: Define Debugging Options

			You can specify debugging options here.

			Defining DEBUG true provides some useful commands:
			CMDS, VARS, FNCS

***************************************************************/
#ifdef __MIPSEL__                          /* thechief: don't define when building for router */
  #undef _FULL_
#else
  #define _FULL_ 1
#endif

#ifdef _FULL_

#define PROG_ERRORS     TRUE   		/* identify serious programming errors */
					/* and print extensive error messages */
					/* This will override messages defined in */
					/* bwb_mes.h, and almost all messages will be in English */
#define DEBUG           TRUE  		/* current debugging */
#define CHECK_RECURSION TRUE 		/* check for recursion violation in expression parser */

#else // _full_

#define PROG_ERRORS     FALSE
#define DEBUG           FALSE  		/* current debugging */
#define CHECK_RECURSION FALSE		/* check for recursion violation in expression parser */

#endif // _full_

#define REDIRECT_STDERR FALSE           /* Redirect stderr to file ERRFILE */

/* the next two caused segfaults when enabled (thechief) */
#define INTENSIVE_DEBUG FALSE           /* old debugging; might be useful later */
#define TEST_BSTRING    FALSE           /* test bstring integrity */


/***************************************************************

	bwbasic.h:	This ends the section of definitions that
			users of bwBASIC will normally need to
			specify. The following are internally defined.

			Note that you may need to set up the default
			FILES command and the default editor below.
			See Part I-G

***************************************************************/


#define MULTISEG_LINES	TRUE		/* allow multi-segment lines delimited by ':' */
#define INTERACTIVE	FALSE 		/* interactive programming environment and related commands */
#define UNIX_CMDS	FALSE   	/* implement Unix-style directory commands  */
#define NUMBER_DOUBLE	FALSE		/* define BASIC number as double: default is float*/

#define COMMAND_SHELL   TRUE		/* allow command shell processing */
#define PROFILE		TRUE		/* interpret profile at beginning */
#define RT_AUTOEXEC     TRUE		/* Run RouterTech autoexec.bas at beginning */
#define STRUCT_CMDS	TRUE		/* commands for structured programming required by full ANSI BASIC */
#define ANSI_FUNCS	TRUE		/* functions required by ANSI full BASIC */

/* GWBASIC and MS compatibility */
#define COMMON_CMDS	TRUE		/* commands common to ANSI full BASIC and GWBASIC */
#define	COMMON_FUNCS	TRUE		/* functions common to GWBASIC and ANSI full BASIC */
#define MS_FUNCS	TRUE		/* Microsoft-specific functions and commands */
#define MS_CMDS		TRUE		/* commands specific to Microsoft GWBASIC (tm) */

/* headers */
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>

/* define number of commands */

#define CMDS_CORE       24              /* number of core commands defined */
#if UNIX_CMDS
#define CMDS_DIR	5
#else
#define CMDS_DIR	0
#endif
#if COMMON_CMDS
#define CMDS_COMMON	22
#else
#define CMDS_COMMON	0
#endif
#if STRUCT_CMDS
#define CMDS_STC	10
#else
#define CMDS_STC	0
#endif
#if INTERACTIVE
#define CMDS_INT	11
#else
#define CMDS_INT	0
#endif
#if MS_CMDS
#define CMDS_MS         5+IMP_CMDCLS+IMP_CMDLOC+IMP_CMDCOLOR
#else
#define CMDS_MS		0
#endif
#if DEBUG
#define CMDS_DEBUG      3               /* number of debugging cmds */
#else
#define CMDS_DEBUG      0               /* no debugging cmds */
#endif
#define COMMANDS        (CMDS_CORE+CMDS_DEBUG+CMDS_DIR+CMDS_COMMON+CMDS_INT+CMDS_MS+CMDS_STC)

/* define number of functions */

#define FUNCS_BASE      12		/* number of basic functions */
#ifdef INTENSIVE_DEBUG
#define FUNCS_DEBUG	1		/* number of debugging functions */
#else
#define FUNCS_DEBUG	0		/* number of debugging functions */
#endif
#if MS_FUNCS
#define FUNCS_MS        (44+IMP_FNCINKEY)
#else
#define FUNCS_MS	0
#endif
#if COMMON_FUNCS
#define FUNCS_COMMON    9
#else
#define FUNCS_COMMON	0
#endif
#if ANSI_FUNCS
#define FUNCS_ANSI	0
#else
#define FUNCS_ANSI	0
#endif
#define FUNCTIONS	(FUNCS_BASE+FUNCS_DEBUG+FUNCS_MS+FUNCS_COMMON+FUNCS_ANSI)

/* Check for inconsistencies */

#if MULTISEG_LINES & STRUCT_CMDS
/* ERROR:  MULTISEG_LINES and STRUCT_CMDS cannot be defined together! */

/*---------------------------------------------------------------------------*/
/* This is apparently because of the conflict between label identifiers      */
/* typically used with the structured commands, and multisegment colons used */
/* with the common BASIC commands.                                           */
/*                                                                           */
/* Or at the least, don't expect to mix both kinds of commands successfully  */
/* in bwBASIC.  (JBV 8/98)                                                   */
/*---------------------------------------------------------------------------*/
#endif

/***************************************************************

	bwbasic.h:	Part I-G: Define User Defaults

			Defining your default editor and files commands
			is a good idea. You must supply the file name
			for the editor to use.  These defaults can be
			changed from inside the program or in your profile
			program by setting the appropriate variables
			shown below.

***************************************************************/

#define DEFVNAME_PROMPT "BWB.PROMPT$"   /* default variable name for PROMPT */
#define DEFVNAME_FILES  "BWB.FILES$"    /* default variable name for FILES */
#define DEFVNAME_COLORS "BWB.COLORS"    /* default variable name for COLORS */

#if RT_AUTOEXEC
#define AUTOEXECNAME	"autoexec.bas" 	   /* Filename for autoexec execution */
#endif

/* added by thechief - 23 May 2008 */
#define MAX_PARMS 18			  /* reduce by 2 for the maximum number of command-line parameters we give access to */
#define DEFVNAME_PARAMC  "argc$"	  /* default name for the count of parameters */
#define DEFVNAME_PARAM_S "argv"           /* default prefix for parameter strings: number and "$" are appended in bwbasic.c */

/* Added by JBV 10/18/97 */
#define DEFVNAME_CMDARG "COMMAND$"         /* default variable name for BASIC program command arg (after program name) */
#define ERRFILE         "err.out"          /* Filename for redirected error messages */
#define PROFILENAME	"profile.bas" 	   /* Filename for profile execution */
#define	TASKS		4		   /* number of tasks available */
#define MAXARGSIZE      128                /* maximum size of argument */

#if 0 /* JBV 8/17/98 */
#define MAXREADLINESIZE 256             /* size of read_line buffer */
#else
#define MAXREADLINESIZE 1024            /* size of read_line buffer */
#endif

#define MAXCMDNAMESIZE  64              /* maximum size for command name */
#define MAXLINENO       32766           /* maximum line number */
#define MAXVARNAMESIZE  40              /* maximum size for variable name */

#if 0 /* JBV 9/4/97 */
#define MAXFILENAMESIZE 40              /* maximum size for file name */
#define MAXSTRINGSIZE   255             /* maximum string length */
#else
#define MAXFILENAMESIZE 255             /* maximum size for file name */
#define MAXSTRINGSIZE   5000            /* maximum string length */
#endif

#define EXECLEVELS      64              /* EXEC stack levels */

#if 0 /* JBV 8/17/98 */
#define MAX_GOLINES     12              /* Maximum # of lines for ON...GOTO statements */
#else
#define MAX_GOLINES     24              /* Maximum # of lines for ON...GOTO statements */
#endif

#define MAX_FARGS       6               /* maximum # arguments to function */
#define MAX_DIMS	64		/* maximum # of dimensions */

#if 0 /* JBV 8/17/98 */
#define ESTACKSIZE      64              /* elements in expression stack */
#else
#define ESTACKSIZE      128             /* elements in expression stack */
#endif

#define XTXTSTACKSIZE   16              /* elements in eXecute TeXT stack */
#define N_OPERATORS     25              /* number of operators defined */
#define N_ERRORS	25		/* number of errors defined */
#define MAX_PRECEDENCE  20              /* highest (last) level of precedence */

#if 0 /* JBV 9/96 */
#define MININTSIZE      -32767          /* minimum integer size */
#define MAXINTSIZE       32767          /* maximum integer size */
#else
#define MININTSIZE      -2147483647     /* minimum integer size */
#define MAXINTSIZE       2147483647     /* maximum integer size */
#endif

#define DEF_SUBSCRIPT   11              /* default subscript */
#define DEF_DEVICES     16              /* default number of devices available */
#define DEF_WIDTH	128		/* default width for devices */
#define PRN_TAB		0x02		/* send TAB followed by col number to output device */
#define COMPRESS_FUNCS  TRUE

/* Derivative definitions */

#if MULTISEG_LINES
#define MARK_LINES	FALSE
#else
#define MARK_LINES	TRUE
#endif

#if DEBUG
#define PERMANENT_DEBUG TRUE
#else
#define PERMANENT_DEBUG FALSE
#endif


/* typedef for BASIC number */

#if NUMBER_DOUBLE
typedef double bnumber;
#else
typedef float bnumber;
#endif

/* define variable types based on last character */

#define STRING          '$'

/* define mathematical operations */

#define MULTIPLY        '*'
#define DIVIDE          '/'
#define ADD             '+'
#define SUBTRACT        '-'
#define ARGUMENT        'A'

/* Operations defined */

#define OP_ERROR        -255		/* operation error (break out) */
#define OP_NULL         0               /* null: operation not defined yet */
#define NUMBER          1               /* number held as internal variable in uvar */
#define CONST_STRING    2               /* string constant */
#define CONST_NUMERICAL 3               /* numerical constant */
#define FUNCTION        4               /* function header */
#define VARIABLE        5               /* external variable pointed to by xvar */
#define PARENTHESIS     6               /* begin parenthetical expression */
#define OP_ADD          7               /* addition sign '+' */
#define OP_SUBTRACT     8               /* subtraction sign '-' */
#define OP_MULTIPLY     9               /* multiplication sign '*' */
#define OP_DIVIDE       10              /* division sign '/' */
#define OP_MODULUS      11              /* modulus "MOD" */
#define OP_EXPONENT     12              /* exponentiation '^' */
#define OP_INTDIVISION  13              /* integer division sign '\' */
#define OP_NEGATION     14              /* negation '-' ??? */
#define OP_STRJOIN      15              /* string join ';' */
#define OP_STRTAB       16              /* string tab ',' */
#define OP_EQUALS       17              /* either logical equal operator */
#define OP_ASSIGN       18              /* assignment operator */
#define OP_NOTEQUAL     20              /* inequality */
#define OP_LESSTHAN     21              /* less than */
#define OP_GREATERTHAN  22              /* greater than */
#define OP_LTEQ         23              /* less than or equal to */
#define OP_GTEQ         24              /* greater than or equal to */
#define OP_NOT          25              /* negation */
#define OP_AND          26              /* conjunction */
#define OP_OR           27              /* disjunction */
#define OP_XOR          28              /* exclusive or */
#define OP_IMPLIES      29              /* implication */
#define OP_EQUIV        30              /* equivalence */
#define OP_TERMINATE    31              /* terminate expression parsing */
#define OP_USERFNC	32		/* user-defined function */

/* Device input/output modes */

#define DEVMODE_AVAILABLE  -1
#define DEVMODE_CLOSED     0
#define DEVMODE_OUTPUT     1
#define DEVMODE_INPUT      2
#define DEVMODE_APPEND     3
#define DEVMODE_RANDOM     4

/* codes for EXEC stack and for function-sub-label lookup table */

#define EXEC_NORM	0
#define	EXEC_GOSUB	1
#define	EXEC_WHILE	2
#define EXEC_FOR        3
#define EXEC_FUNCTION	4
#define EXEC_CALLSUB	5
#define EXEC_IFTRUE	6
#define EXEC_IFFALSE	7
#define EXEC_MAIN	8
#define EXEC_SELTRUE	9
#define EXEC_SELFALSE	10
#define EXEC_LABEL      11
#define EXEC_DO         12
#define EXEC_ON         13

/***************************************************************

	bwbasic.h	Part II: Structures

***************************************************************/

/* Typdef structure for strings under Bywater BASIC */

typedef struct bstr
   {
   /* unsigned int was unsigned char (JBV 9/4/97) */
   unsigned int length;			/* length of string */
   char *sbuffer;			/* pointer to string buffer */
   int rab;				/* is it a random-access buffer? */
#if TEST_BSTRING
   char name[ MAXVARNAMESIZE + 1 ];	/* name for test purposes */
#endif
   } bstring;

/* Structure used for all variables under Bywater BASIC */

struct bwb_variable
   {
   char name[ MAXVARNAMESIZE + 1 ];	/* name */
   int type;                            /* type, i.e., STRING or NUMBER */
#if OLDWAY
   void *array;				/* pointer to array memory */
#endif
   bnumber *memnum;			/* memory for number */
   bstring *memstr;			/* memory for string */
   size_t array_units;			/* total number of units of memory */
   int  *array_sizes;			/* pointer to array of <dimensions>
                                           integers, with sizes of each
                                           dimension */
   int *array_pos;                      /* current position in array */
   int dimensions;                      /* number of dimensions,
                                           0 = not an array */
   struct bwb_variable *next;           /* next variable in chain */
   int common;				/* should this variable be common to chained programs? */
   int preset;				/* preset variable: CLEAR should not alter */
   };

/* Structure to represent program lines under Bywater BASIC */

struct bwb_line
   {
   struct bwb_line *next;               /* pointer to next line in chain */
   int number;                          /* line number */
   char xnum;				/* is there actually a line number? */
   char *buffer;			/* buffer to hold the line */
   int position;                        /* current position in line */
   int lnpos;                           /* line number position in buffer */
   int lnum;                            /* line number read from buffer */
   int cmdpos;                          /* command position in buffer */
   int cmdnum;                          /* number of command in command table
                                           read from buffer */
   int startpos;                        /* start of rest of line read from buffer */
   int marked;                          /* has line been checked yet? */
   };

/* Structure used for all predefined functions under Bywater BASIC */

struct bwb_function
   {
   char name[ MAXVARNAMESIZE + 1 ];     /* name */
   int type;                            /* type, i.e., STRING or NUMBER */
   int arguments;                       /* number of args passed */
   struct bwb_variable * (*vector) ( int argc, struct bwb_variable *argv, int unique_id );  /* vector to function to call */
   struct bwb_function *next;           /* next function in chain */
   int id;                              /* id to identify multiple functions */
   };

/* Structure to represent all command statements under Bywater BASIC */

struct bwb_command
   {
   char name[ MAXCMDNAMESIZE + 1 ];
   struct bwb_line * (*vector) (struct bwb_line *);
   };

/* Structure to define device stack for Bywater BASIC */

struct dev_element
   {
   int mode;                            /* DEVMODE_ item */
   int width;				/* width for output control */
   int col;				/* current column */
   int reclen;                          /* record length for random access */
   int next_record;			/* next record to read/write */
   int loc;				/* location in file */
   int lof;				/* length of file in bytes (JBV) */
   char filename[ MAXFILENAMESIZE + 1 ];/* filename */
   FILE *cfp;                           /* C file pointer for this device */
   char *buffer;			/* pointer to character buffer for random access */
   };

/* Structure to define expression stack elements under Bywater BASIC */

struct exp_ese
   {
   int operation;                       /* operation at this level */
   char type;				/* type of operation at this level:
   					   STRING or NUMBER */
   bstring sval;			/* string */
   bnumber nval;			/* number */
   char string[ MAXSTRINGSIZE + 1 ]; 	/* string for writing */
   struct bwb_variable *xvar;           /* pointer to external variable */
   struct bwb_function *function;       /* pointer to function structure */
   int array_pos[ MAX_DIMS ];		/* array for variable positions */
   int pos_adv;                         /* position advanced in string */
   int rec_pos;                         /* position marker for recursive calls */
   };

/* structure for FUNCTION-SUB lookup table element */

struct fslte
   {
   char *name;
   struct bwb_line *line;
   int code;
   int startpos;			/* starting position in line */
   struct fslte *next;
   struct bwb_variable *local_variable;
   };

/* Structure to define EXEC stack elements */

struct exse
   {
   struct bwb_line *line;		/* line for execution */
   int code;				/* code to note special operations */
   int position;			/* position in line for restore */
   struct bwb_variable *local_variable;	/* local variable chain and current FOR counter */
   struct fslte *curfslt;		/* Current FSLT (JBV 7/98) */
   int single_line_function;		/* Single- or multi-line function (JBV 8/98) */
   struct bwb_variable *calling_variable[ MAX_FARGS ];
   int n_cvs;				/* number of calling variables */

   /*---------------------------------------------*/
   /* Following items were int type (JBV 1/30/98) */
   /*---------------------------------------------*/
   bnumber for_step;			/* STEP value for FOR */
   bnumber for_target;			/* target value for FOR */

   struct bwb_line *while_line;		/* return line for current WHILE */
   struct bwb_line *wend_line;          /* breakout line for current WHILE (or FOR-NEXT) */
   struct exp_ese expression;		/* expression for evaluation by SELECT CASE */
#if MULTISEG_LINES
   struct bwb_line *for_line;           /* top line for FOR-NEXT loop, multisegmented */
   int for_position;                    /* position in top line for FOR-NEXT loop, multisegmented */
#endif
   };

struct xtxtsl
   {
   int  position;
   struct bwb_line l;
   };

extern char progfile[ MAXARGSIZE ];	/* program file */
extern int rescan;			/* program needs to be rescanned */
extern int number;			/* current line number */
extern struct bwb_line *bwb_l;		/* current line pointer */
extern struct bwb_line bwb_start;	/* starting line marker */
extern struct bwb_line bwb_end;		/* ending line marker */
extern struct bwb_line *data_line;      /* current line to read data */
extern int data_pos;			/* position in data_line */
extern struct bwb_variable var_start;	/* variable list start marker */
extern struct bwb_variable var_end;	/* variable list end marker */
extern struct bwb_function fnc_start;	/* function list start marker */
extern struct bwb_function fnc_end;	/* function list end marker */
extern struct fslte fslt_start;		/* function-sub-label lookup table start marker */
extern struct fslte fslt_end;		/* function-sub-label lookup table end marker */
extern int exsc;			/* EXEC stack counter */
extern int expsc;			/* expression stack counter */
extern int xtxtsc;			/* eXecute TeXT stack counter */
extern struct exse *excs;		/* EXEC stack */
extern struct exp_ese *exps;		/* Expression stack */
extern struct xtxtsl *xtxts;		/* Execute Text stack */

extern struct bwb_variable *ed;         /* EDITOR$ variable */
extern struct bwb_variable *fi;         /* FILES$ variable */
extern struct bwb_variable *pr;         /* PROMPT$ variable */
extern struct bwb_variable *im;         /* IMPLEMENTATION$ variable */
extern struct bwb_variable *co;         /* COLORS variable */
extern struct bwb_variable *cm;         /* COMMAND$ variable (JBV 10/20/97) */

/*----------------------------------------------*/
/* Added by JBV 10/20/97 to support CONT/RESUME */
/*----------------------------------------------*/
extern int old_exsc;
extern struct exse old_excs[EXECLEVELS];

/***************************************************************

	bwbasic.h	Part III: Global Data

***************************************************************/

extern char *bwb_ebuf;
extern int bwb_trace;
extern int dim_base;			/* set by OPTION BASE */
extern struct bwb_command bwb_cmdtable[ COMMANDS ];
extern FILE *errfdevice;                /* output device for error messages */
extern int err_line;			/* line in which error occurred */
extern int err_number;			/* number of last error */
extern char err_gosubl[ MAXVARNAMESIZE + 1 ]; /* line for error GOSUB */
extern char *err_table[ N_ERRORS ];	/* table of error messages */
extern int prn_col;
extern struct bwb_function bwb_prefuncs[ FUNCTIONS ];	/* table of predefined functions */

extern struct dev_element *dev_table;   /* table of devices */

/* Operator Structure and Table */

struct bwb_op
   {
   char symbol[ 8 ];                    /* BASIC symbol for the operator */
   int operation;                       /* internal code for the operator */
   int precedence;                      /* level of precedence, 0 = highest */
   };

extern struct bwb_op exp_ops[ N_OPERATORS ]; /* the table itself, filled in in bwb_tbl.c */

/*----------------------------------------------*/
/* Added by JBV 10/20/97 to support CONT/RESUME */
/*----------------------------------------------*/
extern int break_detected;
extern int cont_resume_pending;

/***************************************************************

	bwbasic.h	Part IV: Function Prototypes

***************************************************************/

extern void bwb_mainloop( void );
extern int bwb_callmain( void ); 
extern void *CALLOC0(size_t nelem, size_t elsize, char *str); /* JBV */
extern void FREE0(void *ptr, char *str); /* JBV */
extern void bwb_init( int argc, char **argv );
extern int bwb_fload( FILE *file );
extern int bwb_ladd( char *buffer, int replace );
extern int bwb_findcmd( int argc, int a, struct bwb_line *l );
extern struct bwb_line *bwb_xtxtline( char *buffer );
extern void bwb_iteration( void );
extern void bwb_execline( void );
extern int bwb_gets( char *buffer );
extern int bwb_error( char *message );
extern void break_handler( void );
extern void break_mes( void ); /* Was break_mes( int x ) (JBV 7/98) */
extern void break_grabber( int x ); /* JBV 7/98 */
extern struct bwb_line *bwb_null( struct bwb_line *l );
extern struct bwb_line *bwb_rem( struct bwb_line *l );
extern struct bwb_line *bwb_lerror( struct bwb_line *l );
extern struct bwb_line *bwb_run( struct bwb_line *l );
extern struct bwb_line *bwb_let( struct bwb_line *l );
extern struct bwb_line *bwb_load( struct bwb_line *l );
extern struct bwb_line *bwb_merge( struct bwb_line *l );
extern struct bwb_line *bwb_chain( struct bwb_line *l );
extern struct bwb_line *bwb_common( struct bwb_line *l );
extern int bwb_xload( char *l, int *i );
extern struct bwb_line *bwb_new( struct bwb_line *l );
extern struct bwb_line *bwb_save( struct bwb_line *l );
extern struct bwb_line *bwb_list( struct bwb_line *l );
extern struct bwb_line *bwb_xlist( struct bwb_line *l, FILE *file );
extern struct bwb_line *bwb_cont( struct bwb_line *l );
extern struct bwb_line *bwb_resume( struct bwb_line *l );
extern struct bwb_line *bwb_go( struct bwb_line *l );
extern struct bwb_line *bwb_goto( struct bwb_line *l );
extern struct bwb_line *bwb_gosub( struct bwb_line *l );
extern struct bwb_line *bwb_return( struct bwb_line *l );
extern struct bwb_line *bwb_xend( struct bwb_line *l );
extern struct bwb_line *bwb_system( struct bwb_line *l );
extern struct bwb_line *bwb_tron( struct bwb_line *l );
extern struct bwb_line *bwb_troff( struct bwb_line *l );
extern struct bwb_line *bwb_randomize( struct bwb_line *l );
extern struct bwb_line *bwb_stop( struct bwb_line *l );
extern struct bwb_line *bwb_data( struct bwb_line *l );
extern struct bwb_line *bwb_read( struct bwb_line *l );
extern struct bwb_line *bwb_restore( struct bwb_line *l );
extern struct bwb_line *bwb_delete( struct bwb_line *l );
extern struct bwb_line *bwb_if( struct bwb_line *l );
extern struct bwb_line *bwb_else( struct bwb_line *l );
extern struct bwb_line *bwb_elseif( struct bwb_line *l );
extern struct bwb_line *bwb_select( struct bwb_line *l );
extern struct bwb_line *bwb_case( struct bwb_line *l );
extern struct bwb_line *bwb_endselect( struct bwb_line *l );
extern struct bwb_line *bwb_endif( struct bwb_line *l );
extern struct bwb_line *bwb_while( struct bwb_line *l );
extern struct bwb_line *bwb_wend( struct bwb_line *l );
extern struct bwb_line *bwb_for( struct bwb_line *l );
extern struct bwb_line *bwb_next( struct bwb_line *l );
extern struct bwb_line *bwb_dim( struct bwb_line *l );
extern struct bwb_line *bwb_option( struct bwb_line *l );
extern struct bwb_line *bwb_open( struct bwb_line *l );
extern struct bwb_line *bwb_close( struct bwb_line *l );
extern struct bwb_line *bwb_get( struct bwb_line *l );
extern struct bwb_line *bwb_put( struct bwb_line *l );
extern struct bwb_line *bwb_rmdir( struct bwb_line *l );
extern struct bwb_line *bwb_chdir( struct bwb_line *l );
extern struct bwb_line *bwb_mkdir( struct bwb_line *l );
extern struct bwb_line *bwb_kill( struct bwb_line *l );
extern struct bwb_line *bwb_name( struct bwb_line *l );
extern struct bwb_line *bwb_rset( struct bwb_line *l );
extern struct bwb_line *bwb_lset( struct bwb_line *l );
extern struct bwb_line *bwb_field( struct bwb_line *l );
extern struct bwb_line *bwb_on( struct bwb_line *l );
extern struct bwb_line *bwb_line( struct bwb_line *l );
extern struct bwb_line *bwb_ddbl( struct bwb_line *l );
extern struct bwb_line *bwb_dint( struct bwb_line *l );
extern struct bwb_line *bwb_dsng( struct bwb_line *l );
extern struct bwb_line *bwb_dstr( struct bwb_line *l );
extern struct bwb_line *bwb_mid( struct bwb_line *l );
extern struct bwb_line *bwb_clear( struct bwb_line *l );
extern struct bwb_line *bwb_erase( struct bwb_line *l );
extern struct bwb_line *bwb_swap( struct bwb_line *l );
extern struct bwb_line *bwb_environ( struct bwb_line *l );
extern struct bwb_line *bwb_width( struct bwb_line *l );
extern struct bwb_line *bwb_write( struct bwb_line *l );
extern struct bwb_line *bwb_edit( struct bwb_line *l );
extern struct bwb_line *bwb_files( struct bwb_line *l );
extern struct bwb_line *bwb_do( struct bwb_line *l );
extern struct bwb_line *bwb_doloop( struct bwb_line *l );
extern struct bwb_line *bwb_cls( struct bwb_line *l );
extern struct bwb_line *bwb_locate( struct bwb_line *l );
extern struct bwb_line *bwb_color( struct bwb_line *l );
extern struct bwb_line *bwb_do( struct bwb_line *l );
extern struct bwb_line *bwb_loop( struct bwb_line *l );
extern struct bwb_line *bwb_exit( struct bwb_line *l );
extern struct bwb_line *bwb_exitfor( struct bwb_line *l );
extern struct bwb_line *bwb_exitdo( struct bwb_line *l );

extern struct bwb_line *bwb_zline( struct bwb_line *l );

extern void bwb_incexec( void );
extern void bwb_decexec( void );
extern int bwb_setexec( struct bwb_line *l, int position, int code );
extern int bwb_getcnd( char *lb, char *lhs, char *rhs, char *op, int *n );
extern int bwb_getlhs( char *lb, char *lhs, int *n );
extern int bwb_getop( char *lb, char *op, int *n );
extern int bwb_getrhs( char *lb, char *rhs, int *n );
extern int bwb_evalcnd( char *lhs, char *rhs, char *op );
extern int bwb_isstr( char *b );
extern int eval_int( int l, int r, char *op );
extern int eval_sng( float l, float r, char *op );
extern int eval_dbl( double l, double r, char *op );
extern struct exp_ese *bwb_exp( char *expression, int assignment, int *position );
extern int exp_getvfname( char *source, char *destination );
extern int exp_operation( int entry_level );
extern int inc_esc( void );
extern int dec_esc( void );
extern int fnc_init( int task );
extern struct bwb_function *fnc_find( char *buffer );
extern struct bwb_line *bwb_def( struct bwb_line *l );
extern int bwb_getargs( char *buffer );
extern int bwb_stripcr( char *s );
extern int bwb_numseq( char *buffer, int *start, int *end );
extern int bwb_freeline( struct bwb_line *l );
extern struct bwb_line *bwb_print( struct bwb_line *l );
extern int bwb_xprint( struct bwb_line *l, FILE *f );
extern int bwb_eltype( char *l_buffer, int p );
extern int var_init( int task );
extern int fslt_init( int task );
extern int var_delcvars( void );
extern struct bwb_variable *var_new( char *name );
extern struct bwb_variable *var_islocal( char *buffer );
extern int bwb_strel( char *lb, char *sb, int *n );
extern struct bwb_variable *bwb_numel( char *lb, int *n );
extern int bwb_const( char *lb, char *sb, int *n );
extern int bwb_getvarname( char *lb, char *sb, int *n );
extern struct bwb_variable *var_find( char *buffer );
extern int bwb_isvar( char *buffer );
extern struct bwb_line *bwb_input( struct bwb_line *l );
extern int inp_adv( char *b, int *c );
extern bnumber inp_numconst( char *expression ); /* Was local (JBV 9/98) */
extern int var_make( struct bwb_variable *v, int type );
extern bstring *var_getsval( struct bwb_variable *nvar );
extern bstring *var_findsval( struct bwb_variable *v, int *pp );
extern bstring *exp_getsval( struct exp_ese *e );
extern int dim_getparams( char *buffer, int *pos, int *n_params, int **pp );
extern int adv_element( char *buffer, int *pos, char *element );
extern int adv_ws( char *buffer, int *pos );
#if MULTISEG_LINES
extern int adv_eos( char *buffer, int *pos );
#endif
extern int line_start( char *buffer, int *pos, int *lnpos, int *lnum,
   int *cmdpos, int *cmdnum, int *startpos );
extern int is_cmd( char *buffer, int *cmdnum );
extern int is_let( char *buffer, int *cmdnum );
extern int is_eol( char *buffer, int *position );
extern int is_numconst( char *buffer );
extern int is_label( char *buffer );
extern struct bwb_line * find_label( char *buffer );
extern struct bwb_line *find_loop( struct bwb_line *l );
extern int int_qmdstr( char *buffer_a, char *buffer_b );
extern struct bwb_line * cnd_xpline( struct bwb_line *l, char *buffer );
extern int scan_element( char *buffer, int *pos, char *element );

extern int prn_precision( struct bwb_variable *v );
extern int * prn_getcol( FILE *f );
extern int prn_getwidth( FILE *f );
extern int prn_xprintf( FILE *f, char *buffer );
extern int prn_xxprintf( FILE *f, char *buffer ); /* JBV */
extern int bwb_strtoupper( char *buffer );
extern int bwb_strtolower( char *buffer ); /* JBV 10/98 */
extern int getcmdnum( char *cmdstr );

extern int str_btoc( char *buffer, bstring *s );
extern int str_btob( bstring *d, bstring *s );
extern int str_ctob( bstring *s, char *buffer );
extern int str_cmp( bstring *s, bstring *t );
extern char * str_cat( bstring *s, bstring *t );
extern int exp_findop( char *expression );
extern int exp_isop( char *expression );
extern int exp_isfn( char *expression );
extern int exp_isufn( char *expression );
extern int exp_isnc( char *expression );
extern int exp_isvn( char *expression );
extern int exp_iscmd( char *expression );
extern int exp_paren( char *expression );
extern int exp_strconst( char *expression );
extern int exp_numconst( char *expression );
extern int exp_function( char *expression );
extern int exp_ufnc( char *expression );
extern int exp_variable( char *expression );
extern int exp_validarg( char *expression );
extern int ln_asbuf( struct bwb_line *l, char *s );
extern int xputc( FILE *f, char c );
extern int bwx_signon( void );
extern int bwx_message( char *m );
extern int bwx_putc( char c );
extern int bwx_errmes( char *m );
extern int bwx_input( char *prompt, char *buffer );
extern void bwx_terminate( void );

#if COMMAND_SHELL
extern int bwx_shell( struct bwb_line *l );
#endif

int bwb_scan( void );
struct bwb_line *bwb_call( struct bwb_line *l );
struct bwb_line *bwb_sub( struct bwb_line *l );
struct bwb_line *bwb_endsub( struct bwb_line *l );
struct bwb_line *bwb_endfnc( struct bwb_line *l );
struct bwb_line *bwb_function( struct bwb_line *l );

extern bnumber var_getnval( struct bwb_variable *nvar );
extern bnumber *var_findnval( struct bwb_variable *v, int *pp );
extern bnumber exp_getnval( struct exp_ese *e );
extern bnumber *exp_findnval( struct exp_ese *e );

#if INTERACTIVE
extern int bwb_interact( void );
#endif

#if DEBUG
extern int bwb_debug( char *message );
extern struct bwb_line *bwb_cmds( struct bwb_line *l );
extern struct bwb_line *bwb_vars( struct bwb_line *l );
extern struct bwb_line *bwb_fncs( struct bwb_line *l );
#endif

#ifdef ALLOW_RENUM
extern struct bwb_line *bwb_renum( struct bwb_line *l );
#endif

#if UNIX_CMDS
#if !HAVE_UNISTD /* Not needed if one has <unistd.h> (JBV) */
extern int rmdir( char *path );
extern int chdir( char *path );
#endif
#if !HAVE_SYSSTAT /* Not needed if one has <sys/stat.h> (JBV) */
#if MKDIR_ONE_ARG
extern int mkdir( char *path );
#else
extern int mkdir( char *path, unsigned short permissions );
#endif /* JBV */
#endif
#endif

/* declarations of function commands */

extern struct bwb_variable *fnc_null( int argc, struct bwb_variable *argv, int unique_id );
#if COMPRESS_FUNCS
extern struct bwb_variable *fnc_core( int argc, struct bwb_variable *argv, int unique_id );
#else
extern struct bwb_variable *fnc_abs( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_atn( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_cos( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_log( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_sin( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_sqr( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_sgn( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_int( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_rnd( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_exp( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_tan( int argc, struct bwb_variable *argv, int unique_id );
#endif
extern struct bwb_variable *fnc_tab( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_date( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_time( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_chr( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_mid( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_left( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_right( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_timer( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_val( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_len( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_hex( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_oct( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_cint( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_asc( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_mkd( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_mki( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_mks( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_cvi( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_cvd( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_cvs( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable *fnc_string( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable * fnc_spc( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable * fnc_space( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable * fnc_environ( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable * fnc_pos( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable * fnc_err( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable * fnc_erl( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable * fnc_loc( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable * fnc_lof( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable * fnc_eof( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable * fnc_csng( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable * fnc_instr( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable * fnc_str( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable * fnc_inkey( int argc, struct bwb_variable *argv, int unique_id );
extern struct bwb_variable * fnc_lcase( int argc, struct bwb_variable *argv, int unique_id ); /* JBV 10/98 */
extern struct bwb_variable * fnc_ucase( int argc, struct bwb_variable *argv, int unique_id ); /* JBV 10/98 */
extern struct bwb_variable * fnc_trim( int argc, struct bwb_variable *argv, int unique_id ); /* JBV 10/98 */
extern struct bwb_variable * fnc_ltrim( int argc, struct bwb_variable *argv, int unique_id ); /* JBV 10/98 */
extern struct bwb_variable * fnc_rtrim( int argc, struct bwb_variable *argv, int unique_id ); /* JBV 10/98 */
extern struct bwb_variable * fnc_token( int argc, struct bwb_variable *argv, int unique_id ); /* JBV 10/98 */
extern struct bwb_variable * fnc_pcase( int argc, struct bwb_variable *argv, int unique_id ); /* JBV 10/98 */
extern struct bwb_variable * fnc_isalnum( int argc, struct bwb_variable *argv, int unique_id ); /* JBV 10/98 */
extern struct bwb_variable * fnc_isalpha( int argc, struct bwb_variable *argv, int unique_id ); /* JBV 10/98 */
extern struct bwb_variable * fnc_isascii( int argc, struct bwb_variable *argv, int unique_id ); /* JBV 10/98 */
extern struct bwb_variable * fnc_isblank( int argc, struct bwb_variable *argv, int unique_id ); /* JBV 10/98 */
extern struct bwb_variable * fnc_iscntrl( int argc, struct bwb_variable *argv, int unique_id ); /* JBV 10/98 */
extern struct bwb_variable * fnc_isdigit( int argc, struct bwb_variable *argv, int unique_id ); /* JBV 10/98 */
extern struct bwb_variable * fnc_isgraph( int argc, struct bwb_variable *argv, int unique_id ); /* JBV 10/98 */
extern struct bwb_variable * fnc_islower( int argc, struct bwb_variable *argv, int unique_id ); /* JBV 10/98 */
extern struct bwb_variable * fnc_isprint( int argc, struct bwb_variable *argv, int unique_id ); /* JBV 10/98 */
extern struct bwb_variable * fnc_ispunct( int argc, struct bwb_variable *argv, int unique_id ); /* JBV 10/98 */
extern struct bwb_variable * fnc_isspace( int argc, struct bwb_variable *argv, int unique_id ); /* JBV 10/98 */
extern struct bwb_variable * fnc_isupper( int argc, struct bwb_variable *argv, int unique_id ); /* JBV 10/98 */
extern struct bwb_variable * fnc_isxdigit( int argc, struct bwb_variable *argv, int unique_id ); /* JBV 10/98 */
extern struct bwb_variable * fnc_curdir( int argc, struct bwb_variable *argv, int unique_id ); 

extern bnumber trnc_int( bnumber x );
extern int fnc_checkargs( int argc, struct bwb_variable *argv,
   int min, int max );
extern int ufsc;			/* user function stack counter */

#if DEBUG
extern struct bwb_variable *fnc_test( int argc, struct bwb_variable *argv, int unique_id );
#endif

#if COMPRESS_FUNCS
#define         F_ABS           1
#define         F_ATN           2
#define         F_COS           3
#define         F_EXP           4
#define         F_INT           5
#define         F_LOG           6
#define         F_RND           7
#define         F_SGN           8
#define         F_SIN           9
#define         F_SQR           10
#define         F_TAN           11
#endif
