/***************************************************************

        bwx_tty.c       Environment-dependent implementation
                        for Bywater BASIC Interpreter
			using simple TTY-style input/output

			This file should be used as a template
			for developing more sophisticated
			environment-dependent implementations

    bwBASIC version 3.00 beta
    Copyright (c) 1992-2000, Ted A. Campbell
    Copyright (c) 1995-2000, Jon B. Volkoff
    Copyright (c) 2008, TheChief

    http://www.bwBASIC.com
    email: bwBASIC@bwBASIC.com

    Copyright and Permissions Information:

    All U.S. and international rights are claimed by the authors.

    This software is released under the terms of the GNU General
    Public License (GPL), which is distributed with this software
    in the file "COPYING".  The GPL specifies the terms under
    which users may copy and use the software in this distribution.
    Please note that it rules out the distribution of proprietary
    versions of the software, and it rules out the use of altered
    versions of the software. This means that you cannot remove
    the Bywater BASIC name or copyright information.

    A separate license is available for proprietary or commercial
    distribution, for information on which you should contact
    bwBASIC@bwBASIC.com.

***************************************************************/

/*---------------------------------------------------------------*/
/* NOTE: Modifications marked "JBV" were made by Jon B. Volkoff, */
/* 11/1995 (eidetics@cerf.net).                                  */
/*                                                               */
/* Those additionally marked with "DD" were at the suggestion of */
/* Dale DePriest (daled@cadence.com).                            */
/*---------------------------------------------------------------*/

#include <stdio.h>
#include <math.h> /* struct exception defined here (JBV 11/98) */

#include "bwbasic.h"
#include "bwb_mes.h"

#if HAVE_LONGJUMP /* Revised by JBV (bug found by DD) */
#include <setjmp.h>
#endif

extern int prn_col;
#if HAVE_LONGJUMP /* Revised by JBV (bug found by DD) */
extern jmp_buf mark;
#endif

/***************************************************************

        FUNCTION:       main()

        DESCRIPTION:    As in any C program, main() is the basic
                        function from which the rest of the
                        program is called. Some environments,
			however, provide their own main() functions
			(Microsoft Windows (tm) is an example).
			In these cases, the following code will
			have to be included in the initialization
			function that is called by the environment.

***************************************************************/


int
main( int argc, char **argv )
   {

   bwb_init( argc, argv );

#if HAVE_LONGJUMP /* Revised by JBV (bug found by DD) */
#if INTERACTIVE
   setjmp( mark );
#endif
#endif

   /* main program loop */
   /* AGENDA: need to clean up and regularize these program loops: 
      see the notes on this in bwbasic.c */ 

#ifdef THEOLDWAY
   while( !feof( stdin ) )		/* condition !feof( stdin ) added in v1.11 */
      {
      bwb_iteration();
      }
#else
   bwb_mainloop();
#endif

   bwx_terminate();			/* allow ^D (Unix) exit with grace */

   }



/***************************************************************

        FUNCTION:       bwx_signon()

	DESCRIPTION:    This function prints out the sign-on
			message for bwBASIC.

***************************************************************/

int
bwx_signon( void )
   {
   sprintf( bwb_ebuf, "\r%s %s\n", MES_SIGNON, VERSION );
   prn_xprintf( stdout, bwb_ebuf );
   sprintf( bwb_ebuf, "\r%s\n", MES_COPYRIGHT );
   prn_xprintf( stdout, bwb_ebuf );
   sprintf( bwb_ebuf, "\r%s\n", MES_COPYRIGHT_2 ); /* JBV 1/97 */
   prn_xprintf( stdout, bwb_ebuf );
   sprintf( bwb_ebuf, "\r%s\n", MES_WARNING ); /* JBV 9/98 */
   prn_xprintf( stdout, bwb_ebuf );
#if PERMANENT_DEBUG
   sprintf( bwb_ebuf, "\r%s\n", "Debugging Mode" );
   prn_xprintf( stdout, bwb_ebuf );
#endif
   return TRUE;
   }

/***************************************************************

        FUNCTION:       bwx_message()

	DESCRIPTION:    This function outputs a message to the
			default output device.

***************************************************************/

int
bwx_message( char *m )
   {

#if INTENSIVE_DEBUG
   fprintf( stderr, "<MES>" );
#endif

   prn_xprintf( stdout, m );

   return TRUE;

   }

/***************************************************************

        FUNCTION:       bwx_putc()

	DESCRIPTION:    This function outputs a single character
			to the default output device.

***************************************************************/

int
bwx_putc( char c )
   {

   return fputc( c, stdout );

   }

/***************************************************************

        FUNCTION:       bwx_errmes()

	DESCRIPTION:    This function outputs a message to the
			default error-message device.

***************************************************************/

int
bwx_errmes( char *m )
   {
   static char tbuf[ MAXSTRINGSIZE + 1 ];	/* this memory should be
						   permanent in case of memory
						   overrun errors */

   if (( prn_col != 1 ) && ( errfdevice == stderr ))
      {
      prn_xprintf( errfdevice, "\n" );
      }
   if ( number == 0 )
      {
      sprintf( tbuf, "\n%s: %s\n", ERRD_HEADER, m );
      }
   else
      {
      sprintf( tbuf, "\n%s %d: %s\n", ERROR_HEADER, number, m );
      }

#if INTENSIVE_DEBUG
   fprintf( stderr, "<ERR>" );
#endif

   prn_xprintf( errfdevice, tbuf );

   return TRUE;

   }

/***************************************************************

        FUNCTION:       bwx_input()

	DESCRIPTION:    This function outputs the string pointed
			to by 'prompt', then inputs a character
			string.

***************************************************************/

int
bwx_input( char *prompt, char *buffer )
   {

#if INTENSIVE_DEBUG
   fprintf( stderr, "<INP>" );
#endif

   prn_xprintf( stdout, prompt );

   fgets( buffer, MAXREADLINESIZE, stdin );
   * prn_getcol( stdout ) = 1;			/* reset column */

   return TRUE;

   }

/***************************************************************

        FUNCTION:       bwx_terminate()

	DESCRIPTION:    This function terminates program execution.

***************************************************************/

void
bwx_terminate( void )
   {
   exit( 0 );
   }

/***************************************************************
* thechief:
* function string_replace ()
* simple in situ replacement of "old_string" with "new_string" in "buffer"
*
***************************************************************/
int string_replace (const char *old_string, const char *new_string, char *buffer, int bufsize)
{
const char *p;
char *q, *result;
int i,d, match;

  // catch errors
  if (!buffer) return 0;
  if (!old_string || !new_string) return 0;
  if (strcmp (old_string, new_string) == 0) return 0;
  if (!strstr (buffer, old_string)) return 0;
  if (bufsize < strlen (buffer)-1) return 0;
  
  // proceed
  // allocate sufficient memory
  result = malloc (sizeof(char)*strlen(buffer)*strlen(new_string)/strlen(old_string) * 2);
  for (p = buffer, q = result; *p != NULL ; p++)
  {
    // old_string logic
    for (d=0;d<strlen(old_string);d++)
    {
        if (*(p+d) == *(old_string+d)) match=1;
        else match =0;
    }

    // new_string logic
    if (match)
    {
        for (i=0;i<strlen(new_string);i++)
        {
           *(q+i)=*(new_string+i);
        }
        q=q+i;
        p=p+d-1;
    }
    else // copy unchanged chars
    {
    	*q=*p;
    	q++;
    }
 }
 *q = NULL;
 strncpy (buffer, result, bufsize);
 free (result);
 return 1;
}

/***************************************************************
* thechief:
* function rtvars()
* scan string for variables, and convert them to their values, before executing external
* programs
***************************************************************/
int rtvars (char *org, int bufsize)
{
   struct bwb_variable *v;
   char tbuf[ MAXSTRINGSIZE + 1 ];
   
   /* run through the variable list and print variables */
   for ( v = var_start.next; v != &var_end; v = v->next )
   {
    if (strstr (org, v->name))
    {
      switch ( v->type )
         {
         case STRING:
            str_btoc ( tbuf, var_getsval ( v ) );
    	    string_replace (v->name, tbuf, org, bufsize);
            break;
         case NUMBER:
            break; 
/* we only want to process string variables
#if NUMBER_DOUBLE
	    sprintf( tbuf, "%lf", var_getnval( v ) );
#else
	    sprintf( tbuf, "%f", var_getnval( v ) );
#endif
    	    string_replace (v->name, tbuf, org, bufsize);
            break;
*/            
         default:
            break;
         } // switch
    } // if strstr
   } // for 
   return 0;   
}

/***************************************************************

	FUNCTION:       bwx_shell()

	DESCRIPTION:    This function runs a shell program.

***************************************************************/

#if COMMAND_SHELL

extern int
bwx_shell( struct bwb_line *l )
   {
   static char *s_buffer;
   static int init = FALSE;
   static int position;
   char cmd_buf [512]; // this should be big enough for anything!

   /* get memory for temporary buffer if necessary */

   if ( init == FALSE )
      {
      init = TRUE;

      /* Revised to CALLOC0 pass-thru call by JBV */
      if ( ( s_buffer = (char *) CALLOC0( MAXSTRINGSIZE + 1, sizeof( char ), "bwx_shell" )) == NULL )
	 {
	 bwb_error( err_getmem );
	 return FALSE;
	 }
      }

   /* get the first element and check for a line number */

#if INTENSIVE_DEBUG
   sprintf( bwb_ebuf, "in bwx_shell(): line buffer is <%s>.", l->buffer );
   bwb_debug( bwb_ebuf );
#endif

//****************************************************************
//printf ("thechief #2\n- this is where the external commands are actually executed; we received the command: %s.\n", l->buffer);
//* thechief - check for variables and convert them to their values
     	strcpy (cmd_buf, l->buffer);
 	rtvars (cmd_buf, sizeof (cmd_buf));
 	free (l->buffer);
 	l->buffer = malloc (strlen (cmd_buf) + 1);
 	strcpy (l->buffer, cmd_buf);
//* end: thechief
//****************************************************************
 	
   position = 0;
   adv_element( l->buffer, &position, s_buffer );
   if ( is_numconst( s_buffer ) != TRUE )                  /* not a line number */
      {

#if INTENSIVE_DEBUG
      sprintf( bwb_ebuf, "in bwx_shell(): no line number, command <%s>.",
	 l->buffer );
      bwb_debug( bwb_ebuf );
#endif
      
//  printf ("Command=%s\n", l->buffer);
      if ( system( l->buffer ) == 0 )
	 {
	 return TRUE;
	 }
      else
	 {
	 return FALSE;
	 }
      }

   else                                         /* advance past line number */
      {
      adv_ws( l->buffer, &position );           /* advance past whitespace */

#if INTENSIVE_DEBUG
      sprintf( bwb_ebuf, "in bwx_shell(): line number, command <%s>.",
	 l->buffer );
      bwb_debug( bwb_ebuf );
#endif

      if ( system( &( l->buffer[ position ] ) ) == 0 )
	 {
	 return TRUE;
	 }
      else
	 {
	 return FALSE;
	 }
      }
   }
#endif

/***************************************************************

        FUNCTION:       matherr()

        DESCRIPTION:    This function is called to handle math
                        errors in Bywater BASIC.  It displays
                        the error message, then calls the
                        break_handler() routine.

***************************************************************/

int
matherr( struct exception *except )
   {

   perror( MATHERR_HEADER );
   break_handler();

   return FALSE;
   }

/***************************************************************

        FUNCTION:       bwb_renum()

	DESCRIPTION:    This function implements the BASIC RENUM
			command by shelling out to a default
			renumbering program called "renum".
			Added by JBV 10/95

	SYNTAX:		RENUM

***************************************************************/

#ifdef ALLOW_RENUM

struct bwb_line *
bwb_renum( struct bwb_line *l )
   {
   char tbuf[ MAXSTRINGSIZE + 1 ];
   FILE *loadfile;

   sprintf( tbuf, "renum %s\0", progfile );

#if INTENSIVE_DEBUG
   sprintf( bwb_ebuf, "in bwb_renum(): command line <%s>", tbuf );
   bwb_debug( bwb_ebuf );
#else
   system( tbuf );
#endif

   /* open edited file for read */

   if ( ( loadfile = fopen( progfile, "r" )) == NULL )
      {
      sprintf( bwb_ebuf, err_openfile, progfile );
      bwb_error( bwb_ebuf );

      return bwb_zline( l );
      }

   /* clear current contents */

   bwb_new( l ); /* Relocated by JBV (bug found by DD) */

   /* and (re)load the file into memory */

   bwb_fload( loadfile );


   return bwb_zline( l );
   }
#endif /* ALLOW_RENUM */


