#/bin/sh

CLI="./cm_cli"
TIMEOUT=0

# Reserves the handle
# Arg 1: class (container class, "connection" in most cases)
new() {
	echo "Reserving new handle of class $1"
	$CLI <<-EOF
	<message>
	<from>cli</from>
	<to>manager</to>
	<transaction>
	<type>query</type>
	<key>command/$1/new</key>
	</transaction>
	</message>
	!
	EOF
	sleep $TIMEOUT
	return 0
}

# Starts the group
# Arg 1: group ID
group_begin() {
	echo "Starting up a transaction group $1"
	$CLI <<-EOF
	<message>
	<from>cli</from>
	<to>manager</to>
	<transaction>
	<type>group_begin</type>
	<group>$1</group>
	</transaction>
	</message>
	!
	EOF
	sleep $TIMEOUT
	return 0
}

# Finishes the group
# Arg 1: group ID
group_end() {
	echo "Finishing up a transaction group $1"
	$CLI <<-EOF
	<message>
	<from>cli</from>
	<to>manager</to>
	<transaction>
	<type>group_end</type>
	<group>$1</group>
	</transaction>
	</message>
	!
	EOF
	sleep $TIMEOUT
	return 0
}

# Switches path/changes container's type
# Arg 1: handle
# Arg 2: group ID
# Arg 3: type/path
set_type() {
	echo "Setting type of the container $1 to $3 within group $2"
	$CLI <<-EOF
	<message>
	<from>cli</from>
	<to>$1</to>
	<transaction>
	<type>set</type>
	<group>$2</group>
	<key>settings/type</key>
	<values>
	<row>
	<value>$3</value>
	</row>
	</values>
	</transaction>
	</message>
	!
	EOF
	sleep $TIMEOUT
	return 0
}

# Sets <handle>:<submodule>:settings/<parameter> to <value>
# Arg 1: handle
# Arg 2: group ID
# Arg 3: submodule
# Arg 4: parameter
# Arg 5: value
set_generic() {
	echo "Setting up $3:$4 parameter of container $1 to $5 within group $2"
	$CLI <<-EOF
	<message>
	<from>cli</from>
	<to>$1</to>
	<transaction>
	<type>set</type>
	<group>$2</group>
	<key>$3:settings/$4</key>
	<values>
	<row>
	<value>$5</value>
	</row>
	</values>
	</transaction>
	</message>
	!
	EOF
	sleep $TIMEOUT
	return 0
}

set_flan() {
	echo "Setting up ACCESS CONTROL RULE: BLOCK IP $3 for rule $4 : $5"
	$CLI <<-EOF
	<message>
	<from>cli</from>
	<to>$1</to>
	<transaction>
	<type>set</type>
	<group>$2</group>
	<key>command/$3/$4</key>
	<values>
	<row>
	<value>$5</value>
	</row>
	</values>
	</transaction>
	</message>
	!
	EOF
	sleep $TIMEOUT
	return 0
}

# Sets <handle>:pvc:settings/vpi
# Arg 1: handle
# Arg 2: group ID
# Arg 3: vpi
set_vpi() {
	set_generic $1 $2 "pvc" "vpi" $3
	sleep $TIMEOUT
	return 0
}

# Sets <handle>:pvc:settings/vci
# Arg 1: handle
# Arg 2: group ID
# Arg 3: vci
set_vci() {
	set_generic $1 $2 "pvc" "vci" $3
	sleep $TIMEOUT
	return 0
}

# Sets <handle>:static:settings/ip
# Arg 1: handle
# Arg 2: group ID
# Arg 3: ip
set_ip() {
	set_generic $1 $2 "static" "ip" $3
	sleep $TIMEOUT
	return 0
}

# Sets <handle>:static:settings/netmask
# Arg 1: handle
# Arg 2: group ID
# Arg 3: netmask
set_netmask() {
	set_generic $1 $2 "static" "netmask" $3
	sleep $TIMEOUT
	return 0
}

# Sets <handle>:pppoe:settings/username
# Arg 1: handle
# Arg 2: group ID
# Arg 3: username
set_pppoe_username() {
	set_generic $1 $2 "pppoe" "username" $3
	sleep $TIMEOUT
	return 0
}

# Sets <handle>:pppoe:settings/password
# Arg 1: handle
# Arg 2: group ID
# Arg 3: password
set_pppoe_password() {
	set_generic $1 $2 "pppoe" "password" $3
	sleep $TIMEOUT
	return 0
}

# Sets <handle>:pppoe:settings/mode
# Arg 1: handle
# Arg 2: group ID
# Arg 3: mode
set_pppoe_mode() {
	set_generic $1 $2 "pppoe" "mode" $3
	sleep $TIMEOUT
	return 0
}


# Sets <handle>:fwan:settings/private/nat/state
# Arg 1: handle
# Arg 2: group ID
# Arg 3: state
set_nat() {
        set_generic $1 $2 "fwan" "private/nat" $3
        sleep $TIMEOUT
        return 0
}

# Sets <handle>:fwan:settings/private/spi/state
# Arg 1: handle
# Arg 2: group ID
# Arg 3: state
set_spi() {
        set_generic $1 $2 "fwan" "private/spi" $3
        sleep $TIMEOUT
        return 0
}

# Sets <handle>:fwan:settings/private/mtu/state
# Arg 1: handle
# Arg 2: group ID
# Arg 3: state
set_mtu() {
        set_generic $1 $2 "fwan" "private/mtu" $3
        sleep $TIMEOUT
        return 0
}

# Sets <handle>:fwan:settings/ping/state
# Arg 1: handle
# Arg 2: group ID
# Arg 3: state
set_ping() {
        set_generic $1 $2 "fwan" "ping/state" $3
        sleep $TIMEOUT
        return 0
}

# Sets <handle>:fwan:settings/dmz/state
# Arg 1: handle
# Arg 2: group ID
# Arg 3: state
set_dmz() {
        set_generic $1 $2 "fwan" "dmz/state" $3
        sleep $TIMEOUT
        return 0
}

# Sets <handle>:fwan:settings/dmz/InternalClient
# Arg 1: handle
# Arg 2: group ID
# Arg 3: dmz host
set_dmz_host() {
        set_generic $1 $2 "fwan" "dmz/InternalClient" $3
        sleep $TIMEOUT
        return 0
}

set_block_rule() {
	set_flan "flan" $1 $2 $3 $4
	sleep $TIMEOUT
	return 0
}
