#/bin/sh

CLI="./cm_cli"
TIMEOUT=0

# Reserves the handle
# Arg 1: class (container class, "connection" in most cases)
new() {
	echo "Reserving new handle of class $1"
	$CLI <<-EOF
	<message>
	<from>cli</from>
	<to>manager</to>
	<transaction>
	<type>query</type>
	<key>command/$1/new</key>
	</transaction>
	</message>
	!
	EOF
	sleep $TIMEOUT
	return 0
}

# Starts the group
# Arg 1: group ID
group_begin() {
	echo "Starting up a transaction group $1"
	$CLI <<-EOF
	<message>
	<from>cli</from>
	<to>manager</to>
	<transaction>
	<type>group_begin</type>
	<group>$1</group>
	</transaction>
	</message>
	!
	EOF
	sleep $TIMEOUT
	return 0
}

# Finishes the group
# Arg 1: group ID
group_end() {
	echo "Finishing up a transaction group $1"
	$CLI <<-EOF
	<message>
	<from>cli</from>
	<to>manager</to>
	<transaction>
	<type>group_end</type>
	<group>$1</group>
	</transaction>
	</message>
	!
	EOF
	sleep $TIMEOUT
	return 0
}

# Binds connection from 3 pieces
# Arg 1: group ID
# Arg 2: connection handle
# Arg 3: mux handle
# Arg 4: encaps handle
bind3() {
	echo "Binding $2:$3:$4"
	$CLI <<-EOF
	<message>
	<from>cli</from>
	<to>manager</to>
	<transaction>
	<type>set</type>
	<group>$1</group>
	<key>command/bind</key>
	<values>
	<row>
	<value>$2</value>
	<value>$3</value>
	<value>$4</value>
	</row>
	</values>
	</transaction>
	</message>
	!
	EOF
	sleep $TIMEOUT
	return 0
}

# Switches path/changes container's type
# Arg 1: handle
# Arg 2: group ID
# Arg 3: type/path
set_type() {
	echo "Setting type of the container $1 to $3 within group $2"
	$CLI <<-EOF
	<message>
	<from>cli</from>
	<to>$1</to>
	<transaction>
	<type>set</type>
	<group>$2</group>
	<key>settings/type</key>
	<values>
	<row>
	<value>$3</value>
	</row>
	</values>
	</transaction>
	</message>
	!
	EOF
	sleep $TIMEOUT
	return 0
}

# Switches path/changes container's type
# Arg 1: handle
# Arg 2: group ID
# Arg 3: name/description
set_name() {
	echo "Setting name of the container $1 to $3 within group $2"
	$CLI <<-EOF
	<message>
	<from>cli</from>
	<to>$1</to>
	<transaction>
	<type>set</type>
	<group>$2</group>
	<key>settings/description</key>
	<values>
	<row>
	<value>$3</value>
	</row>
	</values>
	</transaction>
	</message>
	!
	EOF
	sleep $TIMEOUT
	return 0
}

# Sets <handle>:<submodule>:settings/<parameter> to <value>
# Arg 1: handle
# Arg 2: group ID
# Arg 3: submodule
# Arg 4: parameter
# Arg 5: value
set_generic() {
	echo "Setting up $3:$4 parameter of container $1 to $5 within group $2"
	$CLI <<-EOF
	<message>
	<from>cli</from>
	<to>$1</to>
	<transaction>
	<type>set</type>
	<group>$2</group>
	<key>$3:settings/$4</key>
	<values>
	<row>
	<value>$5</value>
	</row>
	</values>
	</transaction>
	</message>
	!
	EOF
	sleep $TIMEOUT
	return 0
}

# Sets <handle>:pvc:settings/vpi
# Arg 1: handle
# Arg 2: group ID
# Arg 3: vpi
set_vpi() {
	set_generic $1 $2 "pvc" "vpi" $3
	sleep $TIMEOUT
	return 0
}

# Sets <handle>:pvc:settings/vci
# Arg 1: handle
# Arg 2: group ID
# Arg 3: vci
set_vci() {
	set_generic $1 $2 "pvc" "vci" $3
	sleep $TIMEOUT
	return 0
}

# Sets <handle>:static:settings/ip
# Arg 1: handle
# Arg 2: group ID
# Arg 3: ip
set_ip() {
	set_generic $1 $2 "static" "ip" $3
	sleep $TIMEOUT
	return 0
}

# Sets <handle>:static:settings/netmask
# Arg 1: handle
# Arg 2: group ID
# Arg 3: netmask
set_netmask() {
	set_generic $1 $2 "static" "netmask" $3
	sleep $TIMEOUT
	return 0
}

# Sets <handle>:pppoe:settings/username
# Arg 1: handle
# Arg 2: group ID
# Arg 3: username
set_pppoe_username() {
	set_generic $1 $2 "pppoe" "username" $3
	sleep $TIMEOUT
	return 0
}

# Sets <handle>:pppoe:settings/password
# Arg 1: handle
# Arg 2: group ID
# Arg 3: password
set_pppoe_password() {
	set_generic $1 $2 "pppoe" "password" $3
	sleep $TIMEOUT
	return 0
}

# Sets <handle>:pppoe:settings/mode
# Arg 1: handle
# Arg 2: group ID
# Arg 3: mode
set_pppoe_mode() {
	set_generic $1 $2 "pppoe" "mode" $3
	sleep $TIMEOUT
	return 0
}
