#/bin/sh

CLI="cm_cli"
TIMEOUT=0

# Create a pvc. For simplicity, we always use 0:x
# Arg 1: port Name
# Arg 2: VPI
create_port() {
	echo "Creating port $1, PVC=0:$2"
	$CLI <<-EOF
	<message><to>security</to><from>cli</from><query><key>status/isloggedin</key></query></message>
	!
	<message><to>security</to><from>cli</from><query><key>status/timeout</key></query></message>
	!
	<message><to>container_manager</to><from>cli</from><set><key>command/port/create</key><value>$1</value></set></message>
	!
	<message><to>logic</to><from>cli</from><set><key>command/connect</key><value>$1:sar</value></set></message>
	!
	<message><to>container_manager</to><from>cli</from><apply/></message>
	!
	<message><to>logic</to><from>cli</from><apply/></message>
	!
	<message><to>$1</to><from>cli</from><set><key>pvc:settings/vpi</key><value>0</value></set></message>
	!
	<message><to>$1</to><from>cli</from><set><key>pvc:settings/vci</key><value>$2</value></set></message>
	!
	<message><to>$1</to><from>cli</from><set><key>pvc:settings/traffic_class</key><value>UBR</value></set></message>
	!
	<message><to>$1</to><from>cli</from><apply/></message>
	!
	EOF
	sleep $TIMEOUT
	return 0
}

# Create a connection.
# Arg 1: connection Name
# Arg 2: Port name to use
# Arg 3: connection type
create_connection_base() {
	echo "Creating default $3 connection $1 on port $2"
	$CLI <<-EOF
	<message><to>security</to><from>cli</from><query><key>status/isloggedin</key></query></message>
	!
	<message><to>security</to><from>cli</from><query><key>status/timeout</key></query></message>
	!
	<message><to>container_manager</to><from>cli</from><set><key>command/connection/create</key><value>$1</value></set></message>
	!
	<message><to>logic</to><from>cli</from><set><key>command/connect</key><value>$1:$2</value></set></message>
	!
	<message><to>container_manager</to><from>cli</from><apply/></message>
	!
	<message><to>logic</to><from>cli</from><apply/></message>
	!
	<message><to>$1</to><from>cli</from><set><key>settings/path</key><value>$3</value></set></message>
	!
	<message><to>$1</to><from>cli</from><apply/></message>
	!
	<message><to>$1</to><from>cli</from><set><key>command/start</key></set></message>
	!
	EOF
	sleep $TIMEOUT
	return 0
}

# Create a static connection
# Arg 1: connection name
# Arg 2: Port name to use
# Arg 3: IP address
# Arg 4: Netmask
# Arg 5: Gateway address
create_static() {
	create_connection_base $1 $2 "static"
	echo "Setting static parameters, IP=$3, Mask=$4, GW=$5"
	$CLI <<-EOF
	<message><to>$1</to><from>cli</from><set><key>rfc2684:settings/encaps</key><value>0</value></set></message>
	!
	<message><to>$1</to><from>cli</from><set><key>static:settings/ip</key><value>$3</value></set></message>
	!
	<message><to>$1</to><from>cli</from><set><key>static:settings/netmask</key><value>$4</value></set></message>
	!
	<message><to>resolver</to><from>cli</from><set><key>settings/gw</key><value>$5</value></set></message>
	!
	<message><to>resolver</to><from>cli</from><set><key>settings/nameserver1</key></set></message>
	!
	<message><to>resolver</to><from>cli</from><set><key>settings/nameserver2</key></set></message>
	!
	<message><to>resolver</to><from>cli</from><set><key>settings/nameserver3</key></set></message>
	!
	<message><to>$1</to><from>cli</from><apply/></message>
	!
	<message><to>resolver</to><from>cli</from><apply/></message>
	!
	EOF
	sleep $TIMEOUT
	return 0
}

# Create a pppoe connection
# Arg 1: connection name
# Arg 2: Port name to use
# Arg 3: Username
# Arg 4: password
create_pppoe() {
	create_connection_base $1 $2 "pppoe"
	echo "Setting PPPoE parameters, User=$3, Pass=$4"
	$CLI <<-EOF
	<message><to>$1</to><from>cli</from><set><key>pppoe:settings/plugin</key><value>pppoe</value></set></message>
	!
	<message><to>$1</to><from>cli</from><set><key>pppoe:settings/username</key><value>$3</value></set></message>
	!
	<message><to>$1</to><from>cli</from><set><key>pppoe:settings/password</key><value>$4</value></set></message>
	!
	<message><to>$1</to><from>cli</from><set><key>pppoe:settings/idle</key></set></message>
	!
	<message><to>$1</to><from>cli</from><set><key>pppoe:settings/maxfail</key><value>10</value></set></message>
	!
	<message><to>$1</to><from>cli</from><set><key>pppoe:settings/mode</key></set></message>
	!
	<message><to>$1</to><from>cli</from><set><key>pppoe:command/apply</key></set></message>
	!
	<message><to>$1</to><from>cli</from><apply/></message>
	!
	EOF
	sleep $TIMEOUT
	return 0
}

# Create a pppoa connection
# Arg 1: connection name
# Arg 2: Port name to use
# Arg 3: Username
# Arg 4: password
create_pppoa() {
	create_connection_base $1 $2 "pppoa"
	echo "Setting PPPoA parameters, User=$3, Pass=$4"
	$CLI <<-EOF
	<message><to>$1</to><from>cli</from><set><key>pppoa:settings/plugin</key><value>pppoa</value></set></message>
	!
	<message><to>aa</to><from>cli</from><set><key>pppoa:settings/encaps</key><value>llc-encaps</value></set></message>
	!
	<message><to>$1</to><from>cli</from><set><key>pppoa:settings/username</key><value>$3</value></set></message>
	!
	<message><to>$1</to><from>cli</from><set><key>pppoa:settings/password</key><value>$4</value></set></message>
	!
	<message><to>$1</to><from>cli</from><set><key>pppoa:settings/idle</key></set></message>
	!
	<message><to>$1</to><from>cli</from><set><key>pppoa:settings/maxfail</key><value>10</value></set></message>
	!
	<message><to>$1</to><from>cli</from><set><key>pppoa:settings/mode</key></set></message>
	!
	<message><to>$1</to><from>cli</from><set><key>pppoa:command/apply</key></set></message>
	!
	<message><to>$1</to><from>cli</from><set><key>pppoa:command/start</key></set></message>
	!
	<message><to>$1</to><from>cli</from><apply/></message>
	!
	EOF
	sleep $TIMEOUT
	return 0
}

# Delete a PVC
# Arg 1: port name
delete_port() {
	echo "Deleting port $1"
	$CLI <<-EOF
	<message><to>security</to><from>cli</from><query><key>status/isloggedin</key></query></message>
	!
	<message><to>security</to><from>cli</from><query><key>status/timeout</key></query></message>
	!
	<message><to>container_manager</to><from>cli</from><set><key>command/port/delete</key><value>$1</value></set></message>
	!
	<message><to>container_manager</to><from>cli</from><apply/></message>
	!
	EOF
	sleep $TIMEOUT
	return 0
}

# Delete a connection
# Arg 1: connection name
delete_connection() {
	echo "Deleting connection $1"
	$CLI <<-EOF
	<message><to>security</to><from>cli</from><query><key>status/isloggedin</key></query></message>
	!
	<message><to>security</to><from>cli</from><query><key>status/timeout</key></query></message>
	!
	<message><to>container_manager</to><from>cli</from><set><key>command/connection/delete</key><value>$1</value></set></message>
	!
	<message><to>container_manager</to><from>cli</from><apply/></message>
	!
	EOF
	sleep $TIMEOUT
	return 0
}
