/* -*- Mode: C; tab-width: 2; indent-tabs-mode: t; c-basic-offset: 2 -*-
 * File: body.c
 *
 * Created: Tue May 27 12:54:17 2003
 *
 * $Id: body.c,v 1.1 2003/12/13 20:59:15 mhassler Exp $
 */

#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include "defines.h"
#include "body.h"
#include "buf.h"

#ifdef DMALLOC
#include <dmalloc.h>
#endif /* DMALLOC */

extern int filter_proxy;
extern int filter_cookies;
extern int filter_java;
extern int filter_activex;
extern int filter_popups;

int filterlen = 15;

struct filter filters[] =
{
	{"<applet", BST_HTML, F_OPEN_APPLET},
	{"<applet", BST_SCRIPT, F_OPEN_APPLET},
	{"</applet>", BST_APPLET, F_CLOSE_APPLET},
	{"<\\/applet>", BST_APPLET, F_CLOSE_APPLET},
	{"<object", BST_HTML, F_OPEN_OBJECT},
	{"<object", BST_SCRIPT, F_OPEN_OBJECT},
	{"</object>", BST_OBJECT, F_CLOSE_OBJECT},
	{"<\\/object>", BST_OBJECT, F_CLOSE_OBJECT},
	{"<script", BST_HTML, F_OPEN_SCRIPT},
	{"</script>", BST_SCRIPT, F_CLOSE_SCRIPT},
	{"<\\/script>", BST_SCRIPT, F_CLOSE_SCRIPT},
	{"window.open", BST_SCRIPT, F_WINDOW_OPEN},
	{"this.open", BST_SCRIPT, F_WINDOW_OPEN},
	{"parent.open", BST_SCRIPT, F_WINDOW_OPEN},
	{"document.cookie", BST_SCRIPT, F_SET_COOKIE},
//	{"<body ", BST_HTML, F_OPEN_BODY},
//	{"onunload", BST_BODYTAG, F_ONUNLOAD},
	{0, 0}
};

void body_dec_chunk_size(struct session *ses, int i)
{
	if(ses->server.chunked)
	{
		ses->server.chunksize -= i;
		if(ses->server.chunksize < 0)
			ses->server.chunksize = 0;
	}
}

int body_read_chunk_size(struct session *ses, int i)
{
	char *end, *start;
	int size;

//	dprintf("Read chunk size:\n'");
//	write(1, &ses->server.in.data[i], min(10, ses->server.in.size-i));
//	dprintf("'\n---\n");

	if(ses->server.in.data[i] == 13)
		i++;
	if(ses->server.in.data[i] == 10)
		i++;
	start = &ses->server.in.data[i];
	size = strtol(start, &end, 16);
//	if(*end == 13 || *end == 10)
	if(end != start) // converted at least something...
	{
		if(size == 0)
		{
			session_close_server(ses);
			ses->server.chunked = 0;
		}
		dprintf("Chunk size = %d\n", size);
		ses->server.chunksize = size;
		while(*end != 10)
			end++;
		i += (end-start);
		return i;
	}
	else // oops, didn't get a chunk size...
	{
		session_close_server(ses);
		return -1;
	}
}

int body_parse_server(struct session *ses)
{
	int i, f, code, len, off = 0, buflen, filter = 1;

	dprintf("Parsing body of the server response (chunk size left = %d)\n", ses->server.chunksize);

/*	if(ses->server.sok < 0)
		return buf_move(&ses->client.out, &ses->server.in,
										min(ses->server.in.size, BUF_SIZE - ses->client.out.size));*/

	if(ses->server.sok < 0)
		filter = 0;

	if(filter)
		buflen = ses->server.in.size - filterlen;
	else
		buflen = ses->server.in.size;

	for(i = 0; i < buflen; i++) // for each character in buf
	{
		if(ses->server.chunked && !ses->server.chunksize) // should be a chunk size?
		{
			i = body_read_chunk_size(ses, i);
			if(i < 0)
				return 0;
			else
				continue;
		}

		if(ses->client.out.size + 1 >= BUF_SIZE) // can accept at least 1 character?
			break;

		if(filter)
		{
			for(f = 0; filters[f].substr; f++) // for each filter
			{
				if(peek_state(&ses->server) == filters[f].state)
				{
					len = strlen(filters[f].substr);
					if(!strncasecmp(&ses->server.in.data[i], filters[f].substr, len))
					{
						code = filters[f].code;

						// <applet
						if(code == F_OPEN_APPLET)
						{
							push_state(&ses->server, BST_APPLET);
							if(filter_java)
							{
								ses->server.skip = 1;
								i += len;
								body_dec_chunk_size(ses, len);
								break;
							}
						}

						// </applet>
						else if(code == F_CLOSE_APPLET)
						{
							pop_state(&ses->server);
							if(filter_java)
							{
								ses->server.skip = 0;
								i += len;
								body_dec_chunk_size(ses, len);
								break;
							}
						}

						// <object
						if(code == F_OPEN_OBJECT)
						{
							push_state(&ses->server, BST_OBJECT);
							if(filter_activex)
							{
								ses->server.skip = 1;
								i += len;
								body_dec_chunk_size(ses, len);
								break;
							}
						}

						// </object>
						else if(code == F_CLOSE_OBJECT)
						{
							pop_state(&ses->server);
							if(filter_activex)
							{
								ses->server.skip = 0;
								i += len;
								body_dec_chunk_size(ses, len);
								break;
							}
						}

						// <script
						else if(code == F_OPEN_SCRIPT)
							push_state(&ses->server, BST_SCRIPT);

						// </script>
						else if(code == F_CLOSE_SCRIPT)
							pop_state(&ses->server);

						// window.open || this.open || parent.open
						else if(code == F_WINDOW_OPEN)
						{
							if(filter_popups)
							{
								char *ptr, stub[] = "window.concat";

								if(ses->client.out.size + off + strlen(stub) >= BUF_SIZE) // cannot fit
									goto remove_and_exit;

								ptr = stub;
								while(*ptr)
								{
									ses->client.out.data[ses->client.out.size + off] = *ptr;
									off++;
									ptr++;
								}
								i += len;
								body_dec_chunk_size(ses, len);
								break;
							}
						}

						// document.cookie
						else if(code == F_SET_COOKIE)
						{
							if(filter_cookies)
							{
								char *ptr, stub[] = "ZappedCookie";

								if(ses->client.out.size + off + strlen(stub) >= BUF_SIZE) // cannot fit
									goto remove_and_exit;

								ptr = stub;
								while(*ptr)
								{
									ses->client.out.data[ses->client.out.size + off] = *ptr;
									off++;
									ptr++;
								}
								i += len;
								body_dec_chunk_size(ses, len);
								break;
							}
						}
					}
				}
			}
		}
		if(!ses->server.skip) // copy one character
		{
			ses->client.out.data[ses->client.out.size + off] = ses->server.in.data[i];
			off++;
		}
		body_dec_chunk_size(ses, 1);
	}

remove_and_exit: // remove handled part from buf
	ses->client.out.size += off;
	buf_remove(&ses->server.in, i);

	return 1;
}
