#*******************************************************************************
#*+--------------------------------------------------------------------------+**
#*|                            ****                                          |**
#*|                            ****                                          |**
#*|                            ******o***                                    |**
#*|                      ********_///_****                                   |**
#*|                      ***** /_//_/ ****                                   |**
#*|                       ** ** (__/ ****                                    |**
#*|                           *********                                      |**
#*|                            ****                                          |**
#*|                            ***                                           |**
#*|                                                                          |**
#*|         Copyright (c) 1998-2005 Texas Instruments Incorporated           |**
#*|                        ALL RIGHTS RESERVED                               |**
#*|                                                                          |**
#*| Permission is hereby granted to licensees of Texas Instruments           |**
#*| Incorporated (TI) products to use this computer program for the sole     |**
#*| purpose of implementing a licensee product based on TI products.         |**
#*| No other rights to reproduce, use, or disseminate this computer          |**
#*| program, whether in part or in whole, are granted.                       |**
#*|                                                                          |**
#*| TI makes no representation or warranties with respect to the             |**
#*| performance of this computer program, and specifically disclaims         |**
#*| any responsibility for any damages, special or consequential,            |**
#*| connected with the use of this program.                                  |**
#*|                                                                          |**
#*+--------------------------------------------------------------------------+**
#*******************************************************************************

# ******************************************************************************
# * FILE PURPOSE:    PSPBoot build configuration include
# ******************************************************************************
# * FILE NAME:       bootcfg.mak
# *
# * DESCRIPTION:     Makefile include for PSPBoot build.
# *
# * (C) Copyright 2005, Texas Instruments, Inc
# *****************************************************************************/

# Use this parameter to specify a board type.
# Valid boards are:
# AR7 Series		: AR7DB, AR7VDB, AR7WRD, AR7VWi, AR7VW, AR7Wi, AR7V, 
# 					  AR7RD and AR7L0
# TNETV1050 Series	: TNETV1050SDB, TNETV1050VDB, TNETV1060SDB_WR
# Cable Modems		: TNETC401B, TNETC620 TNETC520, TNETC420, TNETC405T, TNETC4602MF, TNETC430
# WLAN boards		: WLAN (supporting: TNETWA113vg, TNETWA622, PCI-EVM and
# 										TNETWA123vag)
# TNETV1020 Series	: TNETV1020VDB and TNETWA115VAG
BOARD=AR7WRD

# Use this to specify the endianess of the board. 
# Valid values are LE (Little Endian)and BE (Big Endian)
ENDIAN=LE

# Use this to select the preferred network vehicle.
# Valid options are FTP (for FTP client), TFTP (for TFTP client) 
# and NONE (for neither).
NETWORK_VEHICLE=NONE

# Use this to configure DHCP based network settings
# Valid options are YES or NO.
DHCP=YES

# Use this to select the PC APP Support.
# Valid options are YES or NO. When Selecting this option NETWORK_VEHICHLE
# should be chosen as NONE.
PC_APP=YES

# Use this to select whether Flash File system support is needed or not.
# Valid options are YES or NO.
FFS=NO

# Use this to select whether Secondary Flash File system support is needed or not.
# Valid options are YES or NO. Option ignored if FFS is disabled.
SECONDARY_FFS=NO

# Use this to select whether permanent variables support is needed or not.
# Valid options are YES or NO.
PERMANENT_VARIABLES=NO

# Use this to select your build platform. Valid options are
# WINDOWS 		: Use the Tornado Toolchain. (running on Windows)
# LINUX			: Use the monta-vista tool chain (running on Linux)
PLATFORM=LINUX

# GZip support. Use this to select GNU zip file format support
# Valid options are YES or NO
CONF_OMOD_GZIP=NO

# LZMA support. Use this to select LZMA file format support
# Valid options are YES or NO
CONF_OMOD_LZMA=NO

# 7-Zip support. Use this to select 7-Zip file format support.
# Compressed image must by in flat binary format.
# Valid options are YES or NO
CONF_OMOD_7ZIP=NO

# ELF support. Use this to select ELF file format support
# Valid options are YES or NO
CONF_OMOD_ELF=NO

# TIBINARY support. Use this to select TI proprietary file format support
# Valid options are YES or NO
CONF_OMOD_TIBINARY=YES

# NSP Single image support (Image with header, instead of mtd).
# Valid options are YES or NO. 
CONF_OMOD_NSP_SINGLE_IMAGE=NO

# Dual boot image support. Enables best image selection out of two single images. 
# Available only with NSP single image support. 
# Without this option, Dual option is ignored. Valid options are YES or NO. 
CONF_OMOD_DUAL=NO

# TFTP failover support. Download an image from TFTP if failed to boot.
# Valid options are YES or NO. Available only with TFTP support. Without TFTP
# support this option is ignored.
CONF_OMOD_TFTP_FAILOVER=YES

# Reboot command.
# Valid options are YES or NO
CONF_OCMD_REBOOT=YES

# Version command.
# Valid options are YES or NO
CONF_OCMD_VERSION=YES

# Info command.
# Valid options are YES or NO
CONF_OCMD_INFO=NO

# Fa command.
# Valid options are YES or NO
CONF_OCMD_FA=YES

# Printenv command.
# Valid options are YES or NO
CONF_OCMD_PRINTENV=YES

# Setenv command.
# Valid options are YES or NO
CONF_OCMD_SETENV=YES

# Setperenv command.
# Valid options are YES or NO
CONF_OCMD_SETPERENV=NO

# Unsetenv command.
# Valid options are YES or NO
CONF_OCMD_UNSETENV=YES

# Defragenv command.
# Valid options are YES or NO
CONF_OCMD_DEFRAGENV=YES

# Fmt command.
# Valid options are YES or NO
CONF_OCMD_FMT=YES

# Boot command.
# Valid options are YES or NO
CONF_OCMD_BOOT=YES

# Dm command.
# Valid options are YES or NO
CONF_OCMD_DM=NO

# Oclk command
# Valid options are YES or NO
CONF_OCMD_OCLK=YES

# Help command
# Valid options are YES or NO
CONF_OCMD_HELP=YES

# Ls command. 
# Valid options are YES or NO. Available only with FFS support. Without FFS
# support this option is taken as NO.
CONF_OCMD_LS=YES

# Df command. 
# Valid options are YES or NO. Available only with FFS support. Without FFS
# support this option is taken as NO.
CONF_OCMD_DF=YES

# Cp command.
# Valid options are YES or NO. Available only with FFS support. Without FFS
# support this option is taken as NO.
CONF_OCMD_CP=YES

# Cat command. 
# Valid options are YES or NO. Available only with FFS support. Without FFS
# support this option is taken as NO.
CONF_OCMD_CAT=YES

# Rm command. 
# Valid options are YES or NO. Available only with FFS support. Without FFS
# support this option is taken as NO.
CONF_OCMD_RM=YES

# TFTP command. 
# Valid options are YES or NO. Available only with TFTP support. Without TFTP
# support (NETWORK_VEHICLE != TFTP) this option is taken as NO. When NETWORK_VEHICLE is TFTP and this
# option is NO, it means that TFTP download support (for network bootup) is 
# available, but the 'tftp' command is not available from shell. 
CONF_OCMD_TFTP=NO

# FTP command. 
# Valid options are YES or NO. Available only with FTP support. Without FTP
# support (NETWORK_VEHICLE != FTP) this option is taken as NO. When NETWORK_VEHICLE is FTP and this
# option is NO, it means that FTP engine is present but the 'ftp' command is 
# not available. 
CONF_OCMD_FTP=NO
 
# Flash memory configuration: 
# Select the source file for flash driver from psbl/kernel/flash directory, or use default configuration. 
# SRCS_FLASH_C=flashop_amd_16.c 

# Board hardware configuration: 
# The following section is used to generate configuration header file.
 
# Clock frequencies configuration

CONF_REFXTAL_FREQ    = MHZ(25)
CONF_AFEXTAL_FREQ    = KHZ(35328)
CONF_XTAL3IN_FREQ    = MHZ(25)
CONF_CPU_FREQ        = IS_OHIO_CHIP() ? MHZ(212) : MHZ(150)
CONF_SYS_FREQ        = MHZ(125)


# SDRAM and FLASH Memory sizes configuration

CONF_SDRAM_SZ        = MEG(16)
CONF_FLASH_SZ        = MEG(8)
DUAL_FLASH           = NO

# EMIF configuration


EMIF_ASYNC_WAITCYCLE_MAX_EXT_WAIT    = 0x80
EMIF_ASYNC_WAITCYCLE_WAIT_POL         = 0x1

EMIF_SDRAMCFG_NM                 = 0x1
EMIF_SDRAMCFG_PA                 = 0x0
EMIF_SDRAMCFG_IBANK              = 0x2
EMIF_SDRAMCFG_EBANK              = 0x0
EMIF_SDRAMCFG_PAGESIZE           = 0x1
EMIF_SDRAM_REFRESH_INTERVAL     = 0x1E78

EMIF_ASYNC_BANK_1_SS              = 0x0
EMIF_ASYNC_BANK_1_EW              = 0x0   
EMIF_ASYNC_BANK_1_W_SETUP         = 0x0
EMIF_ASYNC_BANK_1_W_STROBE        = 0xE
EMIF_ASYNC_BANK_1_W_HOLD          = 0x1
EMIF_ASYNC_BANK_1_R_SETUP         = 0x0
EMIF_ASYNC_BANK_1_R_STROBE        = 0xE
EMIF_ASYNC_BANK_1_R_HOLD          = 0x1
EMIF_ASYNC_BANK_1_TA              = 0x0
EMIF_ASYNC_BANK_1_ASIZE           = 0x1 

EMIF_ASYNC_BANK_2_SS              = 0x0
EMIF_ASYNC_BANK_2_EW              = 0x0   
EMIF_ASYNC_BANK_2_W_SETUP         = 0x0
EMIF_ASYNC_BANK_2_W_STROBE        = 0xE
EMIF_ASYNC_BANK_2_W_HOLD          = 0x1
EMIF_ASYNC_BANK_2_R_SETUP         = 0x0
EMIF_ASYNC_BANK_2_R_STROBE        = 0xE
EMIF_ASYNC_BANK_2_R_HOLD          = 0x1
EMIF_ASYNC_BANK_2_TA              = 0x0
EMIF_ASYNC_BANK_2_ASIZE           = 0x2 

EMIF_ASYNC_BANK_3_SS              = 0x0
EMIF_ASYNC_BANK_3_EW              = 0x0   
EMIF_ASYNC_BANK_3_W_SETUP         = 0x0
EMIF_ASYNC_BANK_3_W_STROBE        = 0xE
EMIF_ASYNC_BANK_3_W_HOLD          = 0x1
EMIF_ASYNC_BANK_3_R_SETUP         = 0x0
EMIF_ASYNC_BANK_3_R_STROBE        = 0xE
EMIF_ASYNC_BANK_3_R_HOLD          = 0x1
EMIF_ASYNC_BANK_3_TA              = 0x0
EMIF_ASYNC_BANK_3_ASIZE           = 0x0 

EMIF_ASYNC_BANK_4_SS              = 0x0
EMIF_ASYNC_BANK_4_EW              = 0x0   
EMIF_ASYNC_BANK_4_W_SETUP         = 0x0
EMIF_ASYNC_BANK_4_W_STROBE        = 0xE
EMIF_ASYNC_BANK_4_W_HOLD          = 0x1
EMIF_ASYNC_BANK_4_R_SETUP         = 0x0
EMIF_ASYNC_BANK_4_R_STROBE        = 0xE
EMIF_ASYNC_BANK_4_R_HOLD          = 0x1
EMIF_ASYNC_BANK_4_TA              = 0x0
EMIF_ASYNC_BANK_4_ASIZE           = 0x2 


# MAC ports configuration

CONF_NUM_MAC_PORT    = 1
EXTERNAL_PHY_PORT0   = 0
EXTERNAL_PHY_PORT1   = 1
MAC0_PHY_MASK        = (IS_OHIO_CHIP() ? 0x001E0000 : 0x80000000)
MAC1_PHY_MASK        = 0x001E0000
EXTERNAL_PHY_RESET_GPIO = 17


# Cache configuration

CONF_CACHE           = CACHE_WB


# Miscellaneous configuration    
# Maximum size of environment variables block (or DEFAULT)
ENV_SPACE_SIZE = ( 10 * 1024 )

# Approximate size of secondary file system in bytes. Ignored if secondary FFS not enabled
SECONDARY_FFS_SIZE = ( 512 * 1024 )
