/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2005 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/*******************************************************************************
 * FILE PURPOSE:    AX7 VLYNQ Info File
 *******************************************************************************
 * FILE NAME:       ax7_vlynq_v2u_setup.c
 *
 * DESCRIPTION:     AX7 VLYNQ Setup File for V2U
 * 
 * (C) Copyright 2003, Texas Instruments, Inc
 ******************************************************************************/
#include <linux/config.h>
#include <asm/avalanche/generic/pal.h>

#include <asm/avalanche/generic/pal_vlynq.h>
#include <asm/avalanche/generic/pal_vlynqIoctl.h>  
  
void prom_printf(char *fmt, ...);
void vlynq_delay_wait(Uint32 count);

/* V2U 105 definition */  
#define VLYNQ_TNETV105_REG_OFFSET  0x0C000000  /* Physical address of TNETV105 memory */
#define VLYNQ_TNETV105_REG_SIZE    0x00300000  /* Total size of the ACX111 memory   */
#define TNETV105_VL1_REMOTE_SIZE   0x01000000
#define TNETV105_TX_MAP            0x00000000 

#define AVALANCHE_VL_MEM_SIZE  	   0x04000000

typedef struct
{
    int           id;
    unsigned int  offset;
    unsigned int  size;
    unsigned char remote;

} REGION_CONFIG_T;

typedef struct
{
    char hw_intr_line;
    int  irq;
    PAL_VLYNQ_IRQ_POL_ENUM_T   pol;
    PAL_VLYNQ_IRQ_TYPE_ENUM_T  type;

} IRQ_CONFIG_T;

static REGION_CONFIG_T region_config_105[] = {
{ 0, VLYNQ_TNETV105_REG_OFFSET, VLYNQ_TNETV105_REG_SIZE, 1},
{ 0, AVALANCHE_SDRAM_BASE,    TNETV105_VL1_REMOTE_SIZE,0},
{-1, 0, 0, 0}
	                                 };

/*
  VLYNQ  - [0] usbs_usb20_pls
           [1] usbs_vbus_pls         
           [2] usbh_irq_n
           [3] usbh_smi_n
           [4] usbh_rwakeup
           [5] vlynq1_int_pls
           [6] gpio_intpls_o
           [7] ext_wakeup_event_intlvl_o */

/* table from V5.3.2.6 version
 VECMAP vecmap[] = {  
	             {0,2,VLYNQ_REMOTE_DVC,VLYNQ_INTR_ACTIVE_HIGH,VLYNQ_INTR_PULSED,1},
	             {1,3,VLYNQ_REMOTE_DVC,VLYNQ_INTR_ACTIVE_HIGH,VLYNQ_INTR_PULSED,1},
	             {2,4,VLYNQ_REMOTE_DVC,VLYNQ_INTR_ACTIVE_LOW,VLYNQ_INTR_LEVEL,1},
	             {3,5,VLYNQ_REMOTE_DVC,VLYNQ_INTR_ACTIVE_LOW,VLYNQ_INTR_LEVEL,0},
				 {4,6,VLYNQ_REMOTE_DVC,VLYNQ_INTR_ACTIVE_HIGH,VLYNQ_INTR_LEVEL,0},
				 {5,7,VLYNQ_REMOTE_DVC,VLYNQ_INTR_ACTIVE_HIGH,VLYNQ_INTR_PULSED,0},
				 {6,8,VLYNQ_REMOTE_DVC,VLYNQ_INTR_ACTIVE_HIGH,VLYNQ_INTR_PULSED,0},
				 {7,9,VLYNQ_REMOTE_DVC,VLYNQ_INTR_ACTIVE_HIGH,VLYNQ_INTR_PULSED,0}
                  }; */
static IRQ_CONFIG_T irq_config[] = {
	{ 0, 2,	pal_vlynq_high_irq_pol, pal_vlynq_edge_irq_type},
	{ 1, 3,	pal_vlynq_high_irq_pol, pal_vlynq_edge_irq_type},
	{ 2, 4,	pal_vlynq_low_irq_pol,  pal_vlynq_level_irq_type},  
	/* Not enable those isrs for now
	{ 3, 5,	pal_vlynq_low_irq_pol,  pal_vlynq_level_irq_type},
	{ 4, 6,	pal_vlynq_high_irq_pol, pal_vlynq_level_irq_type},
	{ 5, 7,	pal_vlynq_high_irq_pol, pal_vlynq_edge_irq_type},
	{ 6, 8,	pal_vlynq_high_irq_pol, pal_vlynq_edge_irq_type},
	{ 7, 9,	pal_vlynq_high_irq_pol, pal_vlynq_edge_irq_type},
	*/
	{-1, 0,	pal_vlynq_high_irq_pol, pal_vlynq_level_irq_type}
                                   };

/* To maintain the reference of vlynq configuration across v2u driver load and 
   unload. Not an ideal solution but still cleanup for this release. */
static PAL_VLYNQ_CONFIG_T  vlynq_config;      
static PAL_VLYNQ_DEV_HND   *p_vlynq_dev = NULL;
PAL_VLYNQ_HND       *p_vlynq     = NULL;

int avalanche_105_vlynq_teardown(void)
{
    REGION_CONFIG_T *p_region ;
    IRQ_CONFIG_T    *p_irq    = &irq_config[0];
    int             ret_val   = 0;
    
    p_region = &region_config_105[0];

    while(p_irq->hw_intr_line > -1)
    {
        if(PAL_vlynqUnMapIrq(p_vlynq, p_irq->irq, p_vlynq_dev))
        {
             printk("Failed to clean unmap the irq.\n");
             ret_val = -1;
             break;
        }
        
		PAL_vlynqDisableIrq(p_vlynq, p_irq->irq);

        p_irq++;
    }

    while(p_region->id > -1)
    {
        if(PAL_vlynqUnMapRegion(p_vlynq, p_region->remote, p_region->id, p_vlynq_dev))
        {
             printk("Failed to unmap the regions.\n");
             ret_val = -1;
             break;
        }
  
        p_region++;
    }

    printk("Preparing for vlynq tearing down....\n");
    if(PAL_vlynqIoctl(p_vlynq, 
                      PAL_VLYNQ_IOCTL_MAJOR_VAL(PAL_VLYNQ_IOCTL_PREP_LINK_DOWN), 0))
    {
        printk("Failed to back up the VLYNQ data.\n");
        ret_val = -1;
    }

    return(ret_val);
}

int avalanche_105_vlynq_irq_cfg(void)
{
    IRQ_CONFIG_T    *p_irq    = &irq_config[0];

    while(p_irq->hw_intr_line > -1)
	{
		if(PAL_vlynqSetIrqPol(p_vlynq, p_irq->irq,p_irq->pol))
          return(-1);

        if(PAL_vlynqSetIrqType(p_vlynq, p_irq->irq, p_irq->type))
          return(-1);
		
		if(PAL_vlynqEnableIrq(p_vlynq, p_irq->irq))
          return(-1);

		p_irq++;
	}

    return(0);
}

int avalanche_105_vlynq_re_setup(void)
{
    REGION_CONFIG_T    *p_region ;
    IRQ_CONFIG_T       *p_irq    = &irq_config[0];

    vlynq_delay_wait(0xfffff);
	p_region = &region_config_105[0];

    if(!PAL_vlynqGetLinkStatus(p_vlynq))
    {
        printk("The Link to peer is down.\n");
        return (-1);
    }

    printk("Preparing the VLYNQ, now the Link is up. \n");

    if(PAL_vlynqIoctl(p_vlynq, 
                      PAL_VLYNQ_IOCTL_MAJOR_VAL(PAL_VLYNQ_IOCTL_CLEAR_INTERN_ERR), 0))
    {
        printk("Failed to clear the internal VLYNQ errors.\n");
        return (-1);
    }

    if(PAL_vlynqIoctl(p_vlynq,  
                      PAL_VLYNQ_IOCTL_MAJOR_VAL(PAL_VLYNQ_IOCTL_PREP_LINK_UP), 0))
    {
        printk("Failed to restore the VLYNQ. \n");
        return (-1);
    }

    while(p_region->id > -1)
    {
        if(PAL_vlynqMapRegion(p_vlynq, p_region->remote, p_region->id,
				p_region->offset, p_region->size,
				p_vlynq_dev))
	{
            p_region->id = -1;
            printk("Failed to map the regions.\n");
            return (-1);	
	}

	p_region++;
    }

    while(p_irq->hw_intr_line > -1)
    {
        if(PAL_vlynqMapIrq(p_vlynq, p_irq->hw_intr_line, 
			     p_irq->irq, p_vlynq_dev))
	{
	    p_irq->hw_intr_line = -1;
            printk("Failed to map the irq(s).\n");
	    return(-1);
	}

        p_irq++;
    }

    if(avalanche_105_vlynq_irq_cfg())
    {
        printk("Failed to set irq.\n");
        return (-1);
    }

    return(0);
}

static int reset_flag = 1; /* 1: in reset, 0: out of reset */

void avalanche_105_vlynq_init(void)
{
    REGION_CONFIG_T *p_region;
    IRQ_CONFIG_T *p_irq       = &irq_config[0];
    char dev_name[] = "v2u";
    int instance    = 0;

    prom_printf("Setting up the VLYNQ configuration.\n");

	p_region = &region_config_105[0];
	
    vlynq_config.on_soc    = TRUE;

#if (CONFIG_MIPS_AVALANCHE_VLYNQ_PORT == 0) /* for VLYNQ0 */
    vlynq_config.base_addr = AVALANCHE_LOW_VLYNQ_CONTROL_BASE;
    vlynq_config.local_tx_addr      = PHYS_ADDR(AVALANCHE_LOW_VLYNQ_MEM_MAP_BASE); 
#elif (CONFIG_MIPS_AVALANCHE_VLYNQ_PORT == 1) /* for VLYNQ1 */
    vlynq_config.base_addr = AVALANCHE_HIGH_VLYNQ_CONTROL_BASE;
    vlynq_config.local_tx_addr      = PHYS_ADDR(AVALANCHE_HIGH_VLYNQ_MEM_MAP_BASE); 
#endif 

    vlynq_config.local_clock_dir    = pal_vlynq_clk_out;
    vlynq_config.peer_clock_dir     = pal_vlynq_clk_in;
	vlynq_config.local_clock_div    = 0x1;
    vlynq_config.peer_clock_div     = 0x1;
    
	vlynq_config.local_intr_local   = 0x1;
    vlynq_config.local_intr_vector  = 31;
    vlynq_config.local_intr_enable  = 1;
    vlynq_config.local_int2cfg      = 1;
    vlynq_config.local_intr_pointer = 0x08612400;  /* Do not care due to local_int2cfg == 1 */
    vlynq_config.local_endianness   = pal_vlynq_little_en;

    vlynq_config.local_rtm_cfg_type = no_rtm_cfg;
    vlynq_config.local_rtm_sample_value = 0; /* Do not care as rtm_cfg_type is no_rtm_cfg. */
    vlynq_config.local_tx_fast_path = False;

    vlynq_config.peer_intr_local    = 0x0;
    vlynq_config.peer_intr_vector   = 30;
    vlynq_config.peer_intr_enable   = 1;
    vlynq_config.peer_int2cfg       = 0;
    vlynq_config.peer_intr_pointer  = 0x14; 
    
   	vlynq_config.init_swap_flag 	= False;
  
	vlynq_config.peer_tx_addr       = TNETV105_TX_MAP;   /* using V0 of V2U card always */ 
    vlynq_config.peer_endianness    = pal_vlynq_little_en;

    vlynq_config.peer_rtm_cfg_type  = no_rtm_cfg;
    vlynq_config.peer_rtm_sample_value = 0; /* Do not care as rtm_cfg_type is no_rtm_cfg. */
    vlynq_config.peer_tx_fast_path  = False;


    p_vlynq = PAL_vlynqInit(&vlynq_config);
    if(!p_vlynq)
    {
        prom_printf("Failed to initialize the vlynq @ 0x%08x.\n", 
                     vlynq_config.base_addr);
        prom_printf("The error msg: %s.\n", vlynq_config.error_msg);
        goto av_vlynq_init_fail;
    }

    p_vlynq_dev = PAL_vlynqDevCreate(p_vlynq, dev_name, instance, 64, 1);
    if(!p_vlynq_dev)
    {
        prom_printf("Failed to create the %s%d reference for vlynq.\n", 
                     dev_name, instance);
	goto av_vlynq_dev_fail;
    }

    if(PAL_vlynqAddDevice(p_vlynq, p_vlynq_dev, 1))
    {
        prom_printf("Failed to add %s%d reference into vlynq.\n", 
                     dev_name, instance);
        goto av_vlynq_add_device_fail;
    }

    while(p_region->id > -1)
    {
        if(PAL_vlynqMapRegion(p_vlynq, p_region->remote, p_region->id,
				p_region->offset, p_region->size,
				p_vlynq_dev))
	{
            p_region->id = -1;
            prom_printf("Failed to map regions for %s%d in vlynq.\n", 
                         dev_name, instance);
            goto av_vlynq_map_region_fail;	
	}

	p_region++;
    }

    while(p_irq->hw_intr_line > -1)
    {
        if(PAL_vlynqMapIrq(p_vlynq, p_irq->hw_intr_line, 
			     p_irq->irq, p_vlynq_dev))
	{
	    p_irq->hw_intr_line = -1;
            prom_printf("Failed to map interrupts for %s%d in vlynq.\n",
                         dev_name, instance);
	    goto av_vlynq_map_irq_fail;
	}

        p_irq++;
    }

    if(avalanche_105_vlynq_irq_cfg())
    {
        prom_printf("Failed to set the irq on peer vlynq.\n");
        goto av_vlynq_set_irq_fail;
    }

    prom_printf("Success in setting up the VLYNQ.\n");

    reset_flag = 0;

    return;

av_vlynq_set_irq_fail:
av_vlynq_map_irq_fail:

    prom_printf("Unmapping the VLYNQ IRQ(s).\n");
    p_irq = &irq_config[0];
    while(p_irq->hw_intr_line > -1)
    {
        PAL_vlynqUnMapIrq(p_vlynq, p_irq->irq, p_vlynq_dev);
        p_irq++;
    }


av_vlynq_map_region_fail:

    prom_printf("Un mapping the VLYNQ regions.\n");
	p_region = &region_config_105[0];

    while(p_region->id > -1)
    {
        PAL_vlynqUnMapRegion(p_vlynq, p_region->remote, p_region->id,
			     p_vlynq_dev);
 
        p_region++;
    }

    PAL_vlynqRemoveDevice(p_vlynq, p_vlynq_dev);

av_vlynq_add_device_fail:

    prom_printf("Destroying the VLYNQ device.\n");
    PAL_vlynqDevDestroy(p_vlynq_dev);

av_vlynq_dev_fail:

    prom_printf("Cleaning up.......\n");
    if(PAL_vlynqCleanUp(p_vlynq))
        prom_printf("It is not a Clean shut down...\n");

av_vlynq_init_fail:

}


#if 0 /*Not sure the user need that for v2u */
extern REMOTE_VLYNQ_DEV_RESET_CTRL_FN p_remote_vlynq_dev_reset_ctrl;

/* This function is board specific and should be ported for each board. */
void remote_vlynq_dev_reset_ctrl(unsigned int module_reset_bit,
                                 PAL_SYS_RESET_CTRL_T reset_ctrl)
{

    if(module_reset_bit >= 32)
        return;

    switch(module_reset_bit)
    {
        case 0:
        {
            if(OUT_OF_RESET == reset_ctrl)
            {
                /* We do not want to bring 1130 out of reset, if already done so. */ 
                if(reset_flag == 0)
                    return;
				printf("Un-resetting the remote device.\n");
                if(avalanche_105_vlynq_re_setup( ))
                    prom_printf("Failed to get the peer vlynq up. Please try cold boot.\n");
                else
                    reset_flag = 0;
            }
            else if(IN_RESET == reset_ctrl)
            {
                /* We do not want to put 1130 in reset, if it is already in reset. */
                if(reset_flag == 1)
                    return;

                if(avalanche_105_vlynq_teardown( ))
                     prom_printf("Failed to bring down the peer vlynq. Please try cold boot.\n");
                else
                     reset_flag = 1;
				
                prom_printf("Resetting the remote device.\n");
            }
            else
                ;
        }
        break;

        default:
        break;

    }
}
#endif /* #if 0,  Not sure the user need that for v2u */


void  vlynq_dev_init(void)
{
    volatile unsigned int *reset_base = (unsigned int *) AVALANCHE_RESET_CONTROL_BASE;

    /* Not sure the user need that for v2u
	p_remote_vlynq_dev_reset_ctrl = remote_vlynq_dev_reset_ctrl; */

#if (CONFIG_MIPS_AVALANCHE_VLYNQ_PORT == 0) /* for VLYNQ0 */
    *reset_base &= ~((1 << AVALANCHE_LOW_VLYNQ_RESET_BIT));
	vlynq_delay_wait(20000);
	/* Bring vlynq out of reset if not already done */
    *reset_base |= (1 << AVALANCHE_LOW_VLYNQ_RESET_BIT);
#elif (CONFIG_MIPS_AVALANCHE_VLYNQ_PORT == 1) /* for VLYNQ1 */
    *reset_base &= ~((1 << AVALANCHE_HIGH_VLYNQ_RESET_BIT));
	vlynq_delay_wait(20000);
	/* Bring vlynq out of reset if not already done */
    *reset_base |= (1 << AVALANCHE_HIGH_VLYNQ_RESET_BIT);
#endif 
     
    vlynq_delay_wait(20000); /* Allowing sufficient time to VLYNQ to settle down.*/

	PAL_sysVlynqInit();

    avalanche_105_vlynq_init( );
}
