/*
 * Copyright (C) 2005 Texas Instruments, Inc.  All rights reserved.
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * Texas Instruments 1050 specific setup.
 */
#include <linux/config.h>
#include <asm/mips-boards/prom.h>

#include <asm/avalanche/generic/pal.h>

void avalanche_set_vbus_freq(unsigned int new_vbus_freq);
void vlynq_dev_init(void);
static void avalanche_display_titan_variant( int titan_variant );    
static void avalanche_iomux_enable(UINT32 module);

 /*  IO Multiplexer Control */
#define    IOMUX_GPIO      0    /* GPIO */
#define    IOMUX_LCD       1    /* LCD */
#define    IOMUX_KEYPAD    2    /* Keypad */
#define    IOMUX_PHY       3    /* PHY */
#define    IOMUX_SSP       4    /* SSP */
#define    IOMUX_MARVEL    5    /* MARVEL */

/**********************************************************************
 *  IO Multiplexer Control
 **********************************************************************/

#define TNETV1050_IOMUX_KEYPAD1    TITAN_IOMUX_PIN_SEL_7
#define TNETV1050_IOMUX_KEYPAD2    TITAN_IOMUX_PIN_SEL_8
#define IOMUX_KEYPAD1_VAL	         0x54554000
#define IOMUX_KEYPAD2_VAL          0x00154415

#define TNETV1050_IOMUX_LCD1       TITAN_IOMUX_PIN_SEL_8
#define TNETV1050_IOMUX_LCD2       TITAN_IOMUX_PIN_SEL_9
#define TNETV1050_IOMUX_LCD3       TITAN_IOMUX_PIN_SEL_10
#define IOMUX_LCD1_VAL             0x44000000
#define IOMUX_LCD2_VAL             0x55145515
#define IOMUX_LCD3_VAL             0x00005505

#define TNETV1050_IOMUX_PHY0       TITAN_IOMUX_PIN_SEL_10
#define TNETV1050_IOMUX_PHY1       TITAN_IOMUX_PIN_SEL_11
#define IOMUX_PHY0_VAL		         0x50150000
#define IOMUX_PHY1_VAL             0x00000015

#define TNETV1050_IOMUX_SSP1       TITAN_IOMUX_PIN_SEL_1
#define TNETV1050_IOMUX_SSP2       TITAN_IOMUX_PIN_SEL_5
#define IOMUX_SSP1_VAL             0x00055000
#define IOMUX_SSP2_VAL             0x00000300

#define TNETV1050_IOMUX_GPIO1      TITAN_IOMUX_PIN_SEL_4
#define TNETV1050_IOMUX_GPIO2      TITAN_IOMUX_PIN_SEL_5
#define IOMUX_GPIO1_VAL            0x55000000
#define IOMUX_GPIO2_VAL            0x00001540

#define TNETV1050_IOMUX_MARVEL1    TITAN_IOMUX_PIN_SEL_9
#define TNETV1050_IOMUX_MARVEL2    TITAN_IOMUX_PIN_SEL_10
#define TNETV1050_IOMUX_MARVEL3    TITAN_IOMUX_PIN_SEL_5
#define IOMUX_MARVEL1_VAL          0xAA28AA28
#define IOMUX_MARVEL2_VAL          0x0000AA0A
#define IOMUX_MARVEL3_VAL          0x00000C00
#define IOMUX_MARVEL4_VAL          0x00000400

#if defined( CONFIG_MIPS_AVALANCHE_GLCD ) || defined( CONFIG_MIPS_AVALANCHE_GLCD_MODULE )
static void avalanche_glcd_init_gpio( void );
#endif
#if defined( CONFIG_MIPS_AVALANCHE_LCD ) || defined( CONFIG_MIPS_AVALANCHE_LCD_MODULE )
static void avalanche_lcd_init_mux( void );
#endif

static void avalanche_display_titan_variant( int titan_variant )
{
    char    *type_str;    
    
    /* Display Titan variant */
    switch(titan_variant)
    {
        case TITAN_TYPE_1050:
        type_str = "TNETV1050";
        break;
        
        case TITAN_TYPE_1055:
        type_str = "TNETV1055";
        break;
        
        case TITAN_TYPE_1056:
        type_str = "TNETV1056";
        break;
        
        case TITAN_TYPE_1060:
        type_str = "TNETV1060";
        break;
        
        default:
        type_str = "Unknown";
	      printk("titan_type val is 0x%x\n",titan_variant);
    }

    printk("%s Communication Processor detected\n", type_str );
}

void avalanche_soc_platform_init(void)
{
    volatile PIN_SEL_REG_ARRAY_T *pin_sel_array = (PIN_SEL_REG_ARRAY_T*) AVALANCHE_PIN_SEL_BASE;
    volatile unsigned int *eswitch_control_reg  = (unsigned int*) AVALANCHE_ESWITCH_CONTROL;
    PAL_SYS_Tnetv1050Init clkc;
     
    /* Get entities on the board out of reset. */
    PAL_sysResetCtrl( AVALANCHE_ESWITCH_RESET_BIT, OUT_OF_RESET );
    PAL_sysResetCtrl( AVALANCHE_LOW_EPHY_RESET_BIT, OUT_OF_RESET );
    PAL_sysResetCtrl( AVALANCHE_HIGH_EPHY_RESET_BIT, OUT_OF_RESET );

    /* Initialize GPIO */
    PAL_sysGpioInit();
    
    /* Display Titan chip type/variant */
    avalanche_display_titan_variant( avalanche_get_titan_variant() );    
 
    pin_sel_array->reg[3] &= ~0xff000000;  /*Gpio bits:    b 1111 1111 xxxx xxxx xxxx xxxx xxxx xxxx */
    pin_sel_array->reg[3] |=  0x57000000;  /*Bit setting:  b 0101 0100 xxxx xxxx xxxx xxxx xxxx xxxx */  

    pin_sel_array->reg[4] &= ~0x00003fc0;  /*Gpio bits:    b xxxx xxxx xxxx xxxx xx11 1111 11xx xxxx */
    pin_sel_array->reg[4] |=  0x00001740;  /*Bit setting:  b xxxx xxxx xxxx xxxx xx01 0111 01xx xxxx */

    pin_sel_array->reg[9]  &= ~0xf03f0000;  /*Eth LED bits: b 1111 xxxx xx11 1111 xxxx xxxx xxxx xxxx */
    pin_sel_array->reg[9]  |=  0x50150000;  /*bit setting:  b 0101 xxxx xx01 0101 xxxx xxxx xxxx xxxx */
   
    pin_sel_array->reg[10] &= ~0x3f;        /*Eth LED bits: b xxxx xxxx xxxx xxxx xxxx xxxx xx11 1111 */
    pin_sel_array->reg[10] |=  0x15;        /*bit setting:  b xxxx xxxx xxxx xxxx xxxx xxxx xx01 0101 */ 

    /* Initialize clocks */
    clkc.audclk = AUDCLK_FREQ;
    clkc.refclk = REFCLK_FREQ;
    clkc.altclk = ALTCLK_FREQ;
   
    PAL_sysClkcInit(&clkc);
                                                                    
    /* initialize vbus frequency */
    avalanche_set_vbus_freq(PAL_sysClkcGetFreq(CLKC_VBUS));
    
#if defined (CONFIG_MIPS_AVALANCHE_VLYNQ)
    vlynq_dev_init();
#endif

#if defined( CONFIG_MIPS_AVALANCHE_SSP ) || defined( CONFIG_MIPS_AVALANCHE_SSP_MODULE )
   /* Enable io multiplexer for SSP */
   avalanche_iomux_enable(IOMUX_SSP);
#endif

#if defined( CONFIG_MIPS_AVALANCHE_GLCD ) || defined( CONFIG_MIPS_AVALANCHE_GLCD_MODULE )
    /* Init GLCD GPIOs */
    avalanche_glcd_init_gpio();
#endif

#if defined( CONFIG_MIPS_AVALANCHE_LCD ) || defined( CONFIG_MIPS_AVALANCHE_LCD_MODULE )
    /* Init LCD GPIOs and Muxes */
    avalanche_lcd_init_mux();
#endif    

    /* The board comes up in the two CPMAC mode. */
    *eswitch_control_reg = 0x010000ce;    
#if defined(CONFIG_MIPS_TNETV1060SDB_WR)
    /*Set the marvell in and out of reset */
    PAL_sysGpioCtrl(6, GPIO_PIN, GPIO_OUTPUT_PIN);

    /* Create a reset pulse */
    PAL_sysGpioOutBit(6,0);
    udelay(1000);
    PAL_sysGpioOutBit(6,1);

    /*Put the Ephy0 on reset and MDIO out of reset */   
    PAL_sysResetCtrl( AVALANCHE_MDIO_RESET_BIT, OUT_OF_RESET );
    udelay(1000);

    PAL_sysResetCtrl(AVALANCHE_LOW_EPHY_RESET_BIT,IN_RESET);
	udelay(1000);
	
    /*Get the Eswitch alt rst pin out of reset*/
	  *((volatile UINT32 *)(0xA8642884))|= 0x00000001;
    udelay(1000);

    /*Select lan MII mode and Mio dio mode*/
    *((volatile UINT32 *)(0xA8642880))|= 0x0000001F;
    *((volatile UINT32 *)(0xA8642880))&= 0x00000001;
    udelay(1000);

    /*Set the GPIO's to 2nd function for MII port 0 PIN_SEL_9 Register*/
    *((volatile UINT32 *)(TITAN_IOMUX_PIN_SEL_9))|= 0xAA28AA28;
    udelay(1000);

    /* PIN_SEL_10 Register*/
    *((volatile UINT32 *)(TITAN_IOMUX_PIN_SEL_10))&= 0x00000000;
    *((volatile UINT32 *)(TITAN_IOMUX_PIN_SEL_10))|= 0x0000AA0A;
    udelay(1000);

#endif   
}
                                                                         
#ifdef CONFIG_AVALANCHE_INTC_PACING
unsigned int avalanche_get_intc_input_freq(void)
{
    return PAL_sysClkcGetFreq(CLKC_SYS);
}
#endif

#if defined( CONFIG_MIPS_AVALANCHE_GLCD ) || defined( CONFIG_MIPS_AVALANCHE_GLCD_MODULE )
static void avalanche_glcd_init_gpio( void )
{
   /* Set GPIO 6 & 7 to output */                                    
   PAL_sysGpioCtrl(6, GPIO_PIN, GPIO_OUTPUT_PIN);
   PAL_sysGpioCtrl(7, GPIO_PIN, GPIO_OUTPUT_PIN);
}
#endif

#if defined( CONFIG_MIPS_AVALANCHE_LCD ) || defined( CONFIG_MIPS_AVALANCHE_LCD_MODULE )
static void avalanche_lcd_init_mux( void )
{
  /* enable io multiplexer for LCD */
  avalanche_iomux_enable(IOMUX_LCD);

  /* Set GPIO 6 & 7 to output */ 	
  PAL_sysGpioCtrl(6, GPIO_PIN, GPIO_OUTPUT_PIN);
  PAL_sysGpioCtrl(7, GPIO_PIN, GPIO_OUTPUT_PIN);     
  PAL_sysGpioOutBit( 6, 1 );
  PAL_sysGpioOutBit( 7, 1 );

	/* Set LCD clock to 50 MHz */
  PAL_sysClkcSetFreq(CLKC_LCD,CLK_MHZ(50));

	/* take module out of reset */
  PAL_sysResetCtrl(AVALANCHE_LCD_CTRL_RESET_BIT, OUT_OF_RESET);  
  
}

void lcd_pulse( int is_up )                                                         
{                                                                         
	volatile unsigned int temp; 
  int gpio = is_up ? 7 : 6;
                                                                            
  /* Pulse up */                                                    
  PAL_sysGpioOutBit( gpio, 0 );
                                   
	for(temp = 0; temp < 150 * 1000 * 5;temp++);                                       
                                                                          
  /* Pulse down */                                                  
  PAL_sysGpioOutBit( gpio, 1 );
}
#endif

static void avalanche_iomux_enable(UINT32 module)
{
  UINT32  iomux_value;

  switch ( module )
  {
     case IOMUX_GPIO : 
        REG32_READ( TNETV1050_IOMUX_GPIO1,iomux_value );
        iomux_value |= IOMUX_GPIO1_VAL;
        REG32_WRITE( TNETV1050_IOMUX_GPIO1,iomux_value );
        REG32_READ( TNETV1050_IOMUX_GPIO2,iomux_value );
        iomux_value |= IOMUX_GPIO2_VAL;
        REG32_WRITE( TNETV1050_IOMUX_GPIO2,iomux_value );
        break;

     case IOMUX_LCD : 
        REG32_READ( TNETV1050_IOMUX_LCD1,iomux_value );
        iomux_value |= IOMUX_LCD1_VAL;
        REG32_WRITE( TNETV1050_IOMUX_LCD1,iomux_value );
        REG32_READ( TNETV1050_IOMUX_LCD2,iomux_value );
        iomux_value |= IOMUX_LCD2_VAL;
        REG32_WRITE( TNETV1050_IOMUX_LCD2,iomux_value );
        REG32_READ( TNETV1050_IOMUX_LCD3,iomux_value );
        iomux_value |= IOMUX_LCD3_VAL;
        REG32_WRITE( TNETV1050_IOMUX_LCD3,iomux_value );
        break;

     case IOMUX_KEYPAD :
        REG32_READ( TNETV1050_IOMUX_KEYPAD1,iomux_value );
        iomux_value |= IOMUX_KEYPAD1_VAL;
        REG32_WRITE( TNETV1050_IOMUX_KEYPAD1,iomux_value );
        REG32_READ( TNETV1050_IOMUX_KEYPAD2,iomux_value );
        iomux_value |= IOMUX_KEYPAD2_VAL;
        REG32_WRITE( TNETV1050_IOMUX_KEYPAD2,iomux_value );
        break; 

     case IOMUX_PHY : 
        REG32_READ( TNETV1050_IOMUX_PHY0,iomux_value );
        iomux_value |= IOMUX_PHY0_VAL;
        REG32_WRITE( TNETV1050_IOMUX_PHY0,iomux_value );
        REG32_READ( TNETV1050_IOMUX_PHY1,iomux_value );
        iomux_value |= IOMUX_PHY1_VAL;
        REG32_WRITE( TNETV1050_IOMUX_PHY1,iomux_value );
        break; 

     case IOMUX_SSP:
        REG32_READ( TNETV1050_IOMUX_SSP1,iomux_value );
        iomux_value |= IOMUX_SSP1_VAL;
        REG32_WRITE( TNETV1050_IOMUX_SSP1,iomux_value );
        REG32_READ( TNETV1050_IOMUX_SSP2,iomux_value );
        iomux_value |= IOMUX_SSP2_VAL;
        REG32_WRITE( TNETV1050_IOMUX_SSP2,iomux_value );
        break;

     case IOMUX_MARVEL:
        /* Set the GPIO's to 2nd function for MII port 0 PIN_SEL_9 and PIN_SEL_10 Register*/

        REG32_READ( TNETV1050_IOMUX_MARVEL3,iomux_value );
        iomux_value &= ~IOMUX_MARVEL3_VAL;
        REG32_WRITE( TNETV1050_IOMUX_MARVEL3,iomux_value );
        REG32_READ( TNETV1050_IOMUX_MARVEL3,iomux_value );
        iomux_value |= IOMUX_MARVEL4_VAL;
        REG32_WRITE( TNETV1050_IOMUX_MARVEL3,iomux_value );

        REG32_READ( TNETV1050_IOMUX_MARVEL1,iomux_value );
        iomux_value |= IOMUX_MARVEL1_VAL;
        REG32_WRITE( TNETV1050_IOMUX_MARVEL1,iomux_value );

        REG32_WRITE( TNETV1050_IOMUX_MARVEL2,0x00000000 );
        REG32_READ( TNETV1050_IOMUX_MARVEL2,iomux_value );
        iomux_value |= IOMUX_MARVEL2_VAL;
        REG32_WRITE( TNETV1050_IOMUX_MARVEL2,iomux_value );
        break;  
  }

  return;
}

#if defined( CONFIG_MIPS_AVALANCHE_LCD ) || defined( CONFIG_MIPS_AVALANCHE_LCD_MODULE )
EXPORT_SYMBOL(lcd_pulse);
#endif

