/*
 *
 *   chief_flash_ext.h
 *
 *   TheChief's extensions to export the flash information to userland
 *
 * (C) 2008 www.RouterTech.Org; GPLed
 *
 * $Id: chief_flash_ext.h,v 1.0 2008/06/04 23:41:00 thechief Exp $
 *
 */
#ifndef CHIEF_FLASH_EXT
#define CHIEF_FLASH_EXT 1
#include <linux/proc_fs.h>

#define Chief_Intel 1
#define Chief_AMD 2
#define MAX_CHIEF_FLASH_STR_LENGTH 192

static char chief_flash_ext_var[MAX_CHIEF_FLASH_STR_LENGTH];
static struct proc_dir_entry *chief_flash_ext_proc_dir_entry;

/* write to /proc/flashinfo */
static ssize_t chief_flash_ext_write ( struct file *filp, const char *buff, unsigned long len, void *data )
{
  if (copy_to_user( &chief_flash_ext_var[0], buff, len )) 
  {
    return -EFAULT;
  }
  return len;
}

/* read from /proc/flashinfo */
static int chief_flash_ext_read ( char *page, char **start, off_t off, int count, int *eof, void *data )
{
  int len;
  if (off > 0) 
  {
   *eof = 1;
    return 0;
  }
  len = sprintf (page, "%s\n", &chief_flash_ext_var[0]);
  return len;
}

/* sort out the flash manufacturers */
static int chief_process_cfi ( struct cfi_private *cfi )
{
char *buf = kmalloc (256, GFP_KERNEL);
	if (cfi->mfr == 0x01) {strcat (chief_flash_ext_var, "AMD.\n");} else
	if (cfi->mfr == 0x89) {strcat (chief_flash_ext_var, "INTEL.\n");} else
	if (cfi->mfr == 0x1F) {strcat (chief_flash_ext_var, "ATMEL.\n");} else
	if (cfi->mfr == 0x04) {strcat (chief_flash_ext_var, "FUJITSU (AMD-compat).\n");} else
	if (cfi->mfr == 0x98) {strcat (chief_flash_ext_var, "TOSHIBA (AMD-compat).\n");} else
	if (cfi->mfr == 0xC2) {strcat (chief_flash_ext_var, "MACRONIX (AMD-compat).\n");} else
	if (cfi->mfr == 0x20) {strcat (chief_flash_ext_var, "ST (AMD-compat).\n");} else
	if (cfi->mfr == 0xAD) {strcat (chief_flash_ext_var, "HYUNDAI.\n");} else
	if (cfi->mfr == 0xDA) {strcat (chief_flash_ext_var, "WINBOND.\n");} else
	if (cfi->mfr == 0xEC) {strcat (chief_flash_ext_var, "SAMSUNG.\n");} else
	if (cfi->mfr == 0xB0) {strcat (chief_flash_ext_var, "SHARP.\n");} else
	if (cfi->mfr == 0x10) {strcat (chief_flash_ext_var, "NEC.\n");} else
	if (cfi->mfr == 0x9D) {strcat (chief_flash_ext_var, "PMC.\n");} else
	if (cfi->mfr == 0xBF) {strcat (chief_flash_ext_var, "SST.\n");} else
			      {strcat (chief_flash_ext_var, "Unknown!.\n");}
        sprintf (buf, "Manufacturer_ID=0x%4.4X; Chip_ID=0x%4.4X; Chip_Size=0x%4.4X; Erase_Regions=0x%4.4X", 
						cfi->mfr, 
						cfi->id, 
						cfi->numchips * ((1<<cfi->cfiq->DevSize) * cfi->interleave), 
						cfi->cfiq->NumEraseRegions);
	strcat (chief_flash_ext_var, buf);					
	kfree (buf);
	printk ("%s\n", chief_flash_ext_var); 
}

/* write the flash manufacturer information */
static int write_flash_chip_info ( struct cfi_private *cfi, int MFR )
{
int ret = 0;
     if (MFR == Chief_AMD) 
     { strcpy (chief_flash_ext_var, "Flash type: AMD; Manufacturer="); }
     else 
     { strcpy (chief_flash_ext_var, "Flash type: Intel; Manufacturer="); }
     
     chief_process_cfi (cfi);
     
     chief_flash_ext_proc_dir_entry = create_proc_entry ( "flashinfo", 0644, NULL );
     
     if (chief_flash_ext_proc_dir_entry == NULL) 
     {
       printk (KERN_INFO "flashinfo: Couldn't create proc entry\n");
       return -ENOMEM;
     } 
     else 
     {
      chief_flash_ext_proc_dir_entry->read_proc = chief_flash_ext_read;
      chief_flash_ext_proc_dir_entry->write_proc = chief_flash_ext_write;
      chief_flash_ext_proc_dir_entry->owner = THIS_MODULE;
      printk (KERN_INFO "The Chief's FLASH extensions (www.RouterTech.Org): code loaded.\n");
     }
  return ret;
}

#endif /* CHIEF_FLASH_EXT */
