/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2005 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/


/*******************************************************************************
 * FILE PURPOSE:    Graphical LCD Driver and Window Manager Stack Source
 *******************************************************************************
 * FILE NAME:       glcd_hal_apis.h
 *
 * DESCRIPTION:     Graphical LCD HAL APIs Header file
 *
 * REVISION HISTORY:
 * 09-04-2004      Nakshatra     Initial creation
 * 16-02-2005      Hai Shalom
 *
 * (C) Copyright 2004, Texas Instruments, Inc
 ******************************************************************************/

#ifndef __GLCD_HAL_APIS_H__
#define __GLCD_HAL_APIS_H__

#include "glcd_status.h"
typedef GRAHPICS_STATUS_T GLCD_HAL_STATUS;

#define QVGA_HEIGHT    240
#define QVGA_WIDTH     320

typedef enum PANEL_TYPE {
	QVGA = 0
} PANEL_TYPE_T;

typedef enum PANEL_SHADE 
{
	MONOCROME = 0,
	COLOR_ACTIVE,
    COLOR_PASSIVE,

} PANEL_SHADE_T;

typedef struct DISPLAY_PANEL 
{
	PANEL_TYPE_T  panel_type; /* QVGA */
	INT32         max_bpp;
	INT32         min_bpp;
    UINT16        width;      
    UINT16        height;
	PANEL_SHADE_T panel_shade; 

} DISPLAY_PANEL_T;

typedef struct LCD_CNTRL_CONFIG 
{
	const DISPLAY_PANEL_T  *p_disp_panel;
	INT32             hfp;              /* Horizontal front porch */
	INT32             hbp;              /* Horizontal back porch */
	INT32             hsw;              /* Horizontal Sync Pulse Width */
	INT32             vfp;              /* Vertical front porch */
	INT32             vbp;              /* Vertical back porch */
	INT32             vsw;              /* Vertical Sync Pulse Width */
	INT32             ac_bias;          /* AC Bias Pin Frequency */
	INT32             ac_bias_intrpt;   /* AC Bias Pin Transitions per Interrupt */
	INT32             dma_burst_sz;     /* DMA burst size */
	INT32             bpp;              /* Bits per pixel */
	INT32             fdd;              /* FIFO DMA Request Delay */
	INT32             lcd_clk;          /* LCD clock */
	INT32             pxl_clk;          /* Pixel clock */
	UINT8             tft_alt_mode;     /* TFT Alternative Signal Mapping (Only for active) */
    UINT8             stn_565_mode;     /* 12 Bit Per Pixel (5-6-5) Mode (Only for passive) */
    UINT8             mono_8bit_mode;   /* Mono 8-bit Mode: 1=D0-D7 or 0=D0-D3 */
    UINT8             invert_pxl_clock; /* Invert pixel clock */
    UINT8             invert_line_clock;/* Invert line clock */
    UINT8             invert_frm_clock; /* Invert frame clock  */
    UINT8             sync_edge;        /* Horizontal and Vertical Sync Edge:0=rising 1=falling */
    UINT8             sync_ctrl;        /* Horizontal and Vertical Sync Control: 0=ignore */
    UINT8             raster_order;     /* Raster Data Order Select: 1=Most-to-least 0=Least-to-most */
        
} LCD_CNTRL_CONFIG_T;

typedef enum LCD_CMD 
{
	LCD_GET_AC_BIAS_FREQ = 0,
	LCD_SET_AC_BIAS_FREQ,
	LCD_GET_SCREEN_WIDTH,
	LCD_GET_SCREEN_HEIGHT
} LCD_CMD_T;

typedef enum RASTER_LOAD_MODE 
{
	LOAD_DATA = 1,
	LOAD_PALETTE,
} RASTER_LOAD_MODE_T;

GLCD_HAL_STATUS lcd_hal_init(const LCD_CNTRL_CONFIG_T *cfg);
GLCD_HAL_STATUS lcd_hal_blit(RASTER_LOAD_MODE_T load_mode, const char *p_buf);
GLCD_HAL_STATUS lcd_ioctl(LCD_CMD_T cmd, void *val);

/* Functions for calling the Low Level Hardware Functions */
void hardware_enable(void);
void hardware_disable(void);
int lcd_active(void);
void lcd_enable(void);
void lcd_disable(void);
void tifb_init_dma(void);
void tifb_init_registers(void);

#endif __GLCD_HAL_APIS_H__
