/*------------------------------------------------------------------------------

 Copyright (c) 2003 Texas Instruments Incorporated

 ALL RIGHTS RESERVED



"This computer program is subject to a separate license agreement

 signed by Texas Instruments Incorporated and the licensee, and is

 subject to the restrictions therein. No other rights to reproduce,

 use, or disseminate this computer program, whether in part or in

 whole, are granted."



--------------------------------------------------------------------------------

 Module Name:   Kernel extension in netfilter.New target PPPIDLE 



 Module Purpose:   To caliberate ppp idle timeout upon accepted (marked)
		   PPP inbound traffic 

--------------------------------------------------------------------------------

 Revision History:

 Date and Type of Change.
 21/06/05 -> Creation date

------------------------------------------------------------------------------*/
#include <linux/config.h>
#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/ip.h>
#include <net/icmp.h>
#include <net/ip.h>
#include <net/tcp.h>
#include <net/route.h>
#include <linux/netfilter_ipv4/ip_tables.h>

#define MAX_WAN_CONN 8
#define MAX_INTERFACE_NAME_LEN  10 /* This is for "ppp0" so on */

#define PPP_MANGLE_MARK_MASK	0xA0000000

#if 0
#define DEBUGP printk
#else
#define DEBUGP(format, args...)
#endif

static unsigned int reset_ppp_rx_idle(struct sk_buff **pskb,
			   unsigned int hooknum,
			   const struct net_device *in,
			   const struct net_device *out,
			   const void *targinfo,
			   void *userinfo)
{
	struct net_device *ppp_net_dev = NULL;
	char ppp_intf_name[MAX_INTERFACE_NAME_LEN];	
	unsigned int mark = 0;
	
	mark = (((*pskb)->nfmark - PPP_MANGLE_MARK_MASK) >> 16);

	if ((mark > 0) && (mark <= MAX_WAN_CONN)) {

		sprintf(ppp_intf_name, "ppp%d", (mark - 1));
			
		if ((ppp_net_dev = __dev_get_by_name(ppp_intf_name)) != NULL)
			ppp_net_dev->last_rx = jiffies;
	}
  
	return IPT_CONTINUE;
}

static int check(const char *tablename,
		 const struct ipt_entry *e,
		 void *targinfo,
		 unsigned int targinfosize,
		 unsigned int hook_mask)
{
	return 1;
}

static struct ipt_target ipt_pppidle_reg
= { { NULL, NULL }, "PPPIDLE", reset_ppp_rx_idle, check, NULL, THIS_MODULE };

static int __init init(void)
{
	int index = 0;

	if (ipt_register_target(&ipt_pppidle_reg))
		return -EINVAL;

	return 0;
}

static void __exit fini(void)
{
	ipt_unregister_target(&ipt_pppidle_reg);
}

module_init(init);
module_exit(fini);
MODULE_LICENSE("GPL");
