/* Kernel module to match one of a list of Source IP addresses:  IP addresses are in
   the same place so we can treat them as equal */

#include <linux/module.h>
#include <linux/types.h>
#include <linux/udp.h>
#include <linux/skbuff.h>
#include <linux/ip.h>
#include <linux/netdevice.h>

#include <linux/netfilter_ipv4/ipt_mifnets.h>
#include <linux/netfilter_ipv4/ip_tables.h>

#if 0 
#define duprintf(format, args...) printk(format , ## args)
#else
#define duprintf(format, args...)
#endif

/* Returns 1 if the port is matched by the test, 0 otherwise. */

static inline int
mifnets_match(struct ipt_mifnets *mifnets, char *inif_name, u_int32_t saddr,  u_int32_t daddr)
{
	 /* JAH_TODO...Eventually need to optimize the search here */

	unsigned int i, wildcard, size;
	struct mifnets_net *p;

	
	/* an empty mifnets doesn't match anything*/
	if(!(mifnets->flag & IPT_MIFNETS_ALL))
		return 0;

	if(mifnets->flag & IPT_MIFNETS_IFS) {
		wildcard = 0x80000000;
		for(i = 0; i < mifnets->if_count; i ++) {
			size = mifnets->flag & wildcard? mifnets->name_len[i] : IFNAMSIZ;
			if (!size || !strncmp(inif_name, mifnets->ifnames[i], size))
				goto incl_match;
			wildcard >>= 1;
		}
		return 0;
	}	

	/* Match inclusive networks */
incl_match:
	if(mifnets->flag & IPT_MIFNETS_INCLNETS) {
		p = mifnets->incl_subnets;
		for(i = 0 ; i < mifnets->incl_count; i ++, p++) {
			if(!((saddr^p->addr) & p->mask)) 
				goto excl_match;
		}
		return 0;
	}
    
	/* Match exclusive networks */
excl_match:
	if(mifnets->flag & IPT_MIFNETS_EXCLNETS) {
		p = mifnets->excl_subnets;
		for(i = 0 ; i < mifnets->excl_count; i ++, p++) {
			if(!((saddr^p->addr) & p->mask)) 
				return 0;
		}
	}

	if(mifnets->flag & IPT_MIFNETS_DPRIV) {
		if( (daddr & __constant_htonl(0xffff0000)) == __constant_htonl(0xc0a80000) ||  /*192.168.0.0 */
			(daddr & __constant_htonl(0xff000000)) == __constant_htonl(0xa000000) || /*10.0.0.0 */
  			(daddr & __constant_htonl(0xfff00000)) == __constant_htonl(0xac100000) ||  /*172.16.0.0:172.31.0.0 */ 
  			(daddr & __constant_htonl(0xffff0000)) == __constant_htonl(0xa9fe0000)) /*169.254.0.0 */  			
  			return 1;
		else 
			return 0;
			
	}
	
	return 1;
}


/*-----------------------------------*/
/* Match function                    */
/*-----------------------------------*/
static int
match(const struct sk_buff *skb,
      const struct net_device *in,
      const struct net_device *out,
      const void *matchinfo,
      int offset,
      const void *hdr,
      u_int16_t datalen,
      int *hotdrop)
{
    return (mifnets_match( (struct ipt_mifnets *)matchinfo, 
				in->name, 	(skb->nh.iph)->saddr,  (skb->nh.iph)->daddr));
}

/*---------------------------------------------------------*/
/* checkentry function                                     */
/*                                                         */
/* Called when user tries to insert an entry of this type. */
/*---------------------------------------------------------*/
static int
checkentry(const char *tablename,
	   const struct ipt_ip *ip,
	   void *matchinfo,
	   unsigned int matchsize,
	   unsigned int hook_mask)
{
    /* verify size */
    if (matchsize != IPT_ALIGN(sizeof(struct ipt_mifnets)))
      return 0;
    
    return 1;
}


static struct ipt_match multiifnets_match
= { { NULL, NULL }, "mifnets", &match, &checkentry, NULL, THIS_MODULE };



static int __init init(void)
{
	return ipt_register_match(&multiifnets_match);
}


static void __exit fini(void)
{
	ipt_unregister_match(&multiifnets_match);
}

module_init(init);
module_exit(fini);
MODULE_LICENSE("GPL");
