/*
 * INTEL CONFIDENTIAL
 * Copyright (c) 2002, 2003 Intel Corporation.  All rights reserved.
 * 
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel
 * Corporation or its suppliers or licensors.  Title to the
 * Material remains with Intel Corporation or its suppliers and
 * licensors.  The Material contains trade secrets and proprietary
 * and confidential information of Intel or its suppliers and
 * licensors. The Material is protected by worldwide copyright and
 * trade secret laws and treaty provisions.  No part of the Material
 * may be used, copied, reproduced, modified, published, uploaded,
 * posted, transmitted, distributed, or disclosed in any way without
 * Intel's prior express written permission.
 
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you
 * by disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise. Any license
 * under such intellectual property rights must be express and
 * approved by Intel in writing.
 * 
 * $Workfile: UPnPMicroStack.h
 * $Revision: #1.0.1868.18043
 * $Author:   Intel Corporation, Intel Device Builder
 * $Date:     Wednesday, April 06, 2005
 *
 *
 *
 */
#ifndef __UPnPMicrostack__
#define __UPnPMicrostack__


#include "ILibAsyncSocket.h"

#define UPNP_WEB_SERVER_PORT 55345

/*! \file UPnPMicroStack.h 
	\brief MicroStack APIs for Device Implementation
*/

/*! \defgroup MicroStack MicroStack Module
	\{
*/

struct UPnPDataObject;
struct packetheader;

typedef void* UPnPMicroStackToken;
typedef void* UPnPSessionToken;


/* Complex Type Parsers */


/* Complex Type Serializers */



/* UPnP Stack Management */
UPnPMicroStackToken UPnPCreateMicroStack(void *Chain, const char* FriendlyName,const char* FriendlyName2,const char* FriendlyName3,const char* UDN, const char* SerialNumber, const int NotifyCycleSeconds, const unsigned short PortNum);


void UPnPIPAddressListChanged(UPnPMicroStackToken MicroStackToken);
int UPnPGetLocalPortNumber(UPnPSessionToken token);
int   UPnPGetLocalInterfaceToHost(const UPnPSessionToken UPnPToken);
void* UPnPGetWebServerToken(const UPnPMicroStackToken MicroStackToken);

/* UPnP Set Function Pointers Methods */
extern void (*UPnPFP_PresentationPage) (void* upnptoken,struct packetheader *packet);
extern void (*UPnPFP_OSInfo1_MagicOn) (void* upnptoken);
extern void (*UPnPFP_WANCommonInterfaceConfig_GetCommonLinkProperties) (void* upnptoken);
extern void (*UPnPFP_WANCommonInterfaceConfig_GetTotalBytesReceived) (void* upnptoken);
extern void (*UPnPFP_WANCommonInterfaceConfig_GetTotalBytesSent) (void* upnptoken);
extern void (*UPnPFP_WANCommonInterfaceConfig_GetTotalPacketsReceived) (void* upnptoken);
extern void (*UPnPFP_WANCommonInterfaceConfig_GetTotalPacketsSent) (void* upnptoken);
extern void (*UPnPFP_WANIPConnection_AddPortMapping) (void* upnptoken,char* NewRemoteHost,unsigned short NewExternalPort,char* NewProtocol,unsigned short NewInternalPort,char* NewInternalClient,int NewEnabled,char* NewPortMappingDescription,unsigned int NewLeaseDuration);
extern void (*UPnPFP_WANIPConnection_DeletePortMapping) (void* upnptoken,char* NewRemoteHost,unsigned short NewExternalPort,char* NewProtocol);
extern void (*UPnPFP_WANIPConnection_ForceTermination) (void* upnptoken);
extern void (*UPnPFP_WANIPConnection_GetConnectionTypeInfo) (void* upnptoken);
extern void (*UPnPFP_WANIPConnection_GetExternalIPAddress) (void* upnptoken);
extern void (*UPnPFP_WANIPConnection_GetGenericPortMappingEntry) (void* upnptoken,unsigned short NewPortMappingIndex);
extern void (*UPnPFP_WANIPConnection_GetNATRSIPStatus) (void* upnptoken);
extern void (*UPnPFP_WANIPConnection_GetSpecificPortMappingEntry) (void* upnptoken,char* NewRemoteHost,unsigned short NewExternalPort,char* NewProtocol);
extern void (*UPnPFP_WANIPConnection_GetStatusInfo) (void* upnptoken);
extern void (*UPnPFP_WANIPConnection_RequestConnection) (void* upnptoken);
extern void (*UPnPFP_WANIPConnection_SetConnectionType) (void* upnptoken,char* NewConnectionType);


void UPnPSetDisconnectFlag(UPnPSessionToken token,void *flag);

/* Invocation Response Methods */
void UPnPResponse_Error(const UPnPSessionToken UPnPToken, const int ErrorCode, const char* ErrorMsg);
void UPnPResponseGeneric(const UPnPSessionToken UPnPToken,const char* ServiceURI,const char* MethodName,const char* Params);
void UPnPResponse_OSInfo1_MagicOn(const UPnPSessionToken UPnPToken);
void UPnPResponse_WANCommonInterfaceConfig_GetCommonLinkProperties(const UPnPSessionToken UPnPToken, const char* NewWANAccessType, const unsigned int NewLayer1UpstreamMaxBitRate, const unsigned int NewLayer1DownstreamMaxBitRate, const char* NewPhysicalLinkStatus);
void UPnPResponse_WANCommonInterfaceConfig_GetTotalBytesReceived(const UPnPSessionToken UPnPToken, const unsigned int NewTotalBytesReceived);
void UPnPResponse_WANCommonInterfaceConfig_GetTotalBytesSent(const UPnPSessionToken UPnPToken, const unsigned int NewTotalBytesSent);
void UPnPResponse_WANCommonInterfaceConfig_GetTotalPacketsReceived(const UPnPSessionToken UPnPToken, const unsigned int NewTotalPacketsReceived);
void UPnPResponse_WANCommonInterfaceConfig_GetTotalPacketsSent(const UPnPSessionToken UPnPToken, const unsigned int NewTotalPacketsSent);
void UPnPResponse_WANIPConnection_AddPortMapping(const UPnPSessionToken UPnPToken);
void UPnPResponse_WANIPConnection_DeletePortMapping(const UPnPSessionToken UPnPToken);
void UPnPResponse_WANIPConnection_ForceTermination(const UPnPSessionToken UPnPToken);
void UPnPResponse_WANIPConnection_GetConnectionTypeInfo(const UPnPSessionToken UPnPToken, const char* NewConnectionType, const char* NewPossibleConnectionTypes);
void UPnPResponse_WANIPConnection_GetExternalIPAddress(const UPnPSessionToken UPnPToken, const char* NewExternalIPAddress);
void UPnPResponse_WANIPConnection_GetGenericPortMappingEntry(const UPnPSessionToken UPnPToken, const char* NewRemoteHost, const unsigned short NewExternalPort, const char* NewProtocol, const unsigned short NewInternalPort, const char* NewInternalClient, const int NewEnabled, const char* NewPortMappingDescription, const unsigned int NewLeaseDuration);
void UPnPResponse_WANIPConnection_GetNATRSIPStatus(const UPnPSessionToken UPnPToken, const int NewRSIPAvailable, const int NewNATEnabled);
void UPnPResponse_WANIPConnection_GetSpecificPortMappingEntry(const UPnPSessionToken UPnPToken, const unsigned short NewInternalPort, const char* NewInternalClient, const int NewEnabled, const char* NewPortMappingDescription, const unsigned int NewLeaseDuration);
void UPnPResponse_WANIPConnection_GetStatusInfo(const UPnPSessionToken UPnPToken, const char* NewConnectionStatus, const char* NewLastConnectionError, const unsigned int NewUptime);
void UPnPResponse_WANIPConnection_RequestConnection(const UPnPSessionToken UPnPToken);
void UPnPResponse_WANIPConnection_SetConnectionType(const UPnPSessionToken UPnPToken);


/* State Variable Eventing Methods */
void UPnPSetState_WANCommonInterfaceConfig_PhysicalLinkStatus(UPnPMicroStackToken microstack,char* val);
void UPnPSetState_WANIPConnection_PossibleConnectionTypes(UPnPMicroStackToken microstack,char* val);
void UPnPSetState_WANIPConnection_X_Name(UPnPMicroStackToken microstack,char* val);
void UPnPSetState_WANIPConnection_PortMappingNumberOfEntries(UPnPMicroStackToken microstack,unsigned short val);
void UPnPSetState_WANIPConnection_ExternalIPAddress(UPnPMicroStackToken microstack,char* val);
void UPnPSetState_WANIPConnection_ConnectionStatus(UPnPMicroStackToken microstack,char* val);


/*! \} */
#endif
