/*****************Added by Srikiran************/
/* General functions needed by UPnP
 */

#include "pmlist.h"
#include <pthread.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

void PortMapList_Construct(struct PortMapList *pmlist)
{
}

void PortMapList_Destruct(struct PortMapList *pmlist)
{
	struct PortMap *itr;

	while(pmlist->m_pmap){
                PortMap_delPortForward(pmlist->m_pmap->m_PortMappingProtocol,pmlist->m_pmap->m_RemoteHost,
					 pmlist->m_pmap->m_ExternalIP, pmlist->m_pmap->m_ExternalPort, 
					pmlist->m_pmap->m_InternalClient, pmlist->m_pmap->m_InternalPort);
		itr=pmlist->m_pmap;
		NODE_DEL(pmlist->m_pmap, itr);
		PortMap_Destruct(itr);
		free(itr);
	}
}

int PortMapAdd(struct PortMapList *pmlist, char *RemoteHost, char *Proto, char *ExtIP, int ExtPort,
		char *IntIP, int IntPort, int Enabled, char *Desc, int LeaseDuration)
{

	int fd_socket, fd_proto;
	struct sockaddr_in addr;
	struct PortMap *temp;
	struct PortMap		*itr;
	
	if ((strcmp(Proto,"TCP") == 0) || (strcmp(Proto,"tcp") == 0))
		fd_proto = SOCK_STREAM;
	else
		fd_proto = SOCK_DGRAM;
	
	if ((fd_socket = socket(AF_INET,fd_proto, 0)) == -1)
		printf("Socket Error\n");
//		syslog(LOG_DEBUG, "Socket Error");
	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = INADDR_ANY;
	addr.sin_port = htons(ExtPort);
	/* 
	 * Abhishek : Begin Changes 
	 * Port mapping is described as External port and protocol pair.
	 * We have received a conflicting portmapping entry if 
	 * 1. Portmap is SAME and
	 * 2. Internal client using the portmap is different
	 * for such a case we return error code 718.
	 */	
	for (itr = pmlist->m_pmap; itr;)
	{
		if ((itr->m_ExternalPort == ExtPort) 
			&& (strcmp(itr->m_PortMappingProtocol, Proto) == 0)
			&& (strcmp(itr->m_InternalClient, IntIP) != 0))
		{
			/**/
			return(718);
		
		} else {
                                itr = itr->next;
		}
	}
	/* Abhishek : End Changes */	

	if (bind(fd_socket, (struct sockaddr *) &addr, sizeof(addr)) == -1)
	{
		close(fd_socket);
		perror("Port in use\n");
//		syslog(LOG_DEBUG,"Error binding socket");
		return (718);
	}
	close (fd_socket);
	
//	temp = new PortMap;
	temp = (struct PortMap*)malloc(sizeof(struct PortMap));
	//PortMap_Construct(temp);
	
	if (RemoteHost)
   {
		temp->m_RemoteHost=strdup(RemoteHost);
//                temp->m_RemoteHost = new char[strlen(RemoteHost)+1];
//                strcpy(temp->m_RemoteHost, RemoteHost);
   } else temp->m_RemoteHost = NULL;

   if (Proto)
	{
		temp->m_PortMappingProtocol=strdup(Proto);
//		temp->m_PortMappingProtocol = new char[strlen(Proto)+1];
//		strcpy(temp->m_PortMappingProtocol, Proto);
	}
	else temp->m_PortMappingProtocol = NULL;

        if (ExtIP)
        {	
				temp->m_ExternalIP=strdup(ExtIP);
//          temp->m_ExternalIP = new char[strlen(ExtIP)+1];
//          strcpy(temp->m_ExternalIP, ExtIP);
        }
        else temp->m_ExternalIP = NULL;

        temp->m_ExternalPort = ExtPort;

        if (IntIP)
        {
				temp->m_InternalClient=strdup(IntIP);
//          temp->m_InternalClient = new char[strlen(IntIP)+1];
//          strcpy(temp->m_InternalClient, IntIP);
        }
        else temp->m_InternalClient = NULL;

        temp->m_InternalPort = IntPort;

        if (Desc)
        {
				temp->m_PortMappingDescription=strdup(Desc);
//          temp->m_PortMappingDescription = new char[strlen(Desc)+1];
//          strcpy(temp->m_PortMappingDescription, Desc);
        }
        else temp->m_PortMappingDescription = NULL;

   temp->m_PortMappingLeaseDuration = LeaseDuration;

//	m_pmap.push_back(temp);
	NODE_ENQUEUE(pmlist->m_pmap, temp);
	
	PortMap_addPortForward(Proto, RemoteHost, ExtIP, ExtPort, IntIP, IntPort, Enabled, Desc);
	
	return (1);
}


int PortMapDelete(struct PortMapList *pmlist, char *Proto, int ExtPort)
{
	struct PortMap *itr;
	
//	for (list<PortMap *>::iterator itr = m_pmap.begin(); itr != m_pmap.end(); itr++)
	/* Change this to use a double link list */
	for (itr = pmlist->m_pmap; itr; itr=itr->next)
	{
		if (( strcmp(itr->m_PortMappingProtocol,Proto) == 0 ) && (itr->m_ExternalPort == ExtPort))
		{
			PortMap_delPortForward(itr->m_PortMappingProtocol, itr->m_RemoteHost,
						 itr->m_ExternalIP, itr->m_ExternalPort,
						 itr->m_InternalClient,	itr->m_InternalPort);
			
//			delete itr;
//			m_pmap.erase(itr);
			NODE_DEL(pmlist->m_pmap, itr);
			PortMap_Destruct(itr);
			free(itr);
			return (1);
		}
	}
	return 0;
}


int PortMap_addPortForward(char *Proto, char *RemoteHost,char *ExtIP, int ExtPort, 
	char *IntIP,int IntPort, int Enabled, char *Desc)
{
	char command[255];
	char *errstr;
	int retval;

	/*
	 * check for the Remote Host String. if RemoteHost is non-NULL and not "" add the rule with the src IP address
	 * else add the rule without source IP Address
	 */
	if ((RemoteHost != NULL) && (strlen(RemoteHost) > 0)) {
			sprintf(command, IPTABLES " -t nat -A PREROUTING -p %s -s %s -d %s --dport %d -j DNAT --to %s:%d", Proto, RemoteHost, ExtIP, ExtPort, IntIP, IntPort);
	}
	else {
		sprintf(command, IPTABLES " -t nat -A PREROUTING -p %s -d %s --dport %d -j DNAT --to %s:%d", Proto, ExtIP, ExtPort, IntIP, IntPort);
	}
	retval = system(command);
	if (retval) {
		errstr = strerror(retval);
	}
	if ((RemoteHost != NULL) && (strlen(RemoteHost) > 0)) {
		sprintf(command, IPTABLES " -I FORWARD -p %s -s %s -d %s --dport %d -j ACCEPT", Proto, RemoteHost, IntIP, IntPort);
	}
	else {
		sprintf(command, IPTABLES " -I FORWARD -p %s -d %s --dport %d -j ACCEPT", Proto, IntIP, IntPort);
	}
	retval = system(command);
	if (retval) {
		errstr = strerror(retval);
	}

	return (1);
}

int PortMap_delPortForward(char *Proto, char *RemoteHost,char *ExtIP, int ExtPort, 
	char* IntIP, int IntPort)
{
	char command[255];
	char *errstr;
	int retval;
	

	/*
	 * check for the Remote Host String. if RemoteHost is non-NULL and not "" delete the rule using src IP address
	 * else delete the rule without source IP Address
	 */
	if ((RemoteHost != NULL) && (strlen(RemoteHost) > 0)) {
		sprintf(command, IPTABLES  " -t nat -D PREROUTING -p %s -s %s -d %s --dport %d -j DNAT --to %s:%d", Proto, RemoteHost , ExtIP, ExtPort, IntIP, IntPort);
	} else {
		sprintf(command, IPTABLES  " -t nat -D PREROUTING -p %s -d %s --dport %d -j DNAT --to %s:%d", Proto, ExtIP, ExtPort, IntIP, IntPort);
	}
	retval = system(command);	
	if (retval) {
		errstr = strerror(retval);
	}
	if ((RemoteHost != NULL) && (strlen(RemoteHost) > 0)) {
		sprintf(command, IPTABLES  " -D FORWARD -p %s -s %s -d %s --dport %d -j ACCEPT", Proto, RemoteHost, IntIP, IntPort);
	} else {
		sprintf(command, IPTABLES  " -D FORWARD -p %s -d %s --dport %d -j ACCEPT", Proto, IntIP, IntPort);
	}
	retval = system(command);	
	if (retval) {
		errstr = strerror(retval);
	}
	return (1);
}


/*****************Added by Srikiran************/
